#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Module for shared methods
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2025 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


import json
from jinja2 import Template, DictLoader, Environment
from os import environ as env

from actinia_module_plugin.core.templates.user_templates import readTemplate
from actinia_module_plugin.core.template_parameters import (
    get_not_needed_params,
    get_template_undef,
)
from actinia_module_plugin.resources.logging import log
from actinia_module_plugin.resources.templating import pcTplEnv


ENV = {
    key.replace("TEMPLATE_VALUE_", ""): val
    for key, val in env.items()
    if key.startswith("TEMPLATE_VALUE_")
}


def start_job(timeout, func, *args):
    """
    Execute the provided function in a subprocess

    Args:
        timeout: Timeout parameter
        func: The function to call from the subprocess
        *args: The function arguments

    Returns:
        returns after starting the process
    """
    # Just starting the process
    from multiprocessing import Process

    p = Process(target=func, args=args)
    p.start()

    return


def filter_func(name):
    """filter examples out of template folder"""

    if "example" not in name:
        return True
    return False


def get_user_template(name):
    actinia_template = readTemplate(name)
    if actinia_template is False:
        return False
    tpl = Template(json.dumps(actinia_template))

    return tpl


def get_user_template_source(name):
    actinia_template = readTemplate(name)
    tplEnv = Environment(loader=DictLoader({name: actinia_template}))
    tpl_source = tplEnv.loader.get_source(tplEnv, name)[0]

    return tpl_source


def get_global_template_path(name):
    tplPath = name + ".json"

    # change path to template if in subdir
    for i in pcTplEnv.list_templates(filter_func=filter_func):
        if i.split("/")[-1] == tplPath:
            tplPath = i

    return tplPath


def get_global_template(name):
    tplPath = get_global_template_path(name)
    tpl = pcTplEnv.get_template(tplPath)
    return tpl


def get_global_template_source(name):
    tplPath = get_global_template_path(name)
    tpl_source = pcTplEnv.loader.get_source(pcTplEnv, tplPath)[0]
    return tpl_source


def check_for_errors(undef, parsed_content, tpl_source, kwargs):
    """
    This method checks if all placeholders are filled with values and
    returns the placeholder if missing. Exceptions are default values for which
    the given default value can be used and if statements for which the value
    can be empty.
    """
    # find default variables from processchain and variables which are only in
    # an if statement and has not to be set
    not_needed_vars = get_not_needed_params(undef, tpl_source, parsed_content)

    for i in undef:
        # check if undef variables are needed or set in the kwargs
        if i not in kwargs.keys() and i not in not_needed_vars:
            log.error('Required parameter "' + i + '" not in process chain!')
            return i

    return None


def fill_env_values(filled_params, undef):
    """
    This function checks if a undefined variable is set in the environment
    variables and set it in kwargs if not already set.
    """
    if len(ENV) > 0:
        for param in undef:
            if param not in filled_params and param.upper() in ENV:
                filled_params[param] = ENV[param.upper()]


def fillTemplateFromProcessChain(actiniamodulename, kwargs):
    """
    This method receives a process chain name for an actinia module and
    kwargs to fill the template values. It loads the according process
    chain template from kvdb or filesystem. The received values will be
    replaced to be passed to actinia. In case the template has more
    placeholder values than it receives, the missing attribute is
    returned as string.
    """

    pc = actiniamodulename
    tpl_source = ""

    # first see if a user template exists
    tpl = get_user_template(pc)
    tpl_source = get_user_template_source(pc)
    if tpl is False:
        # then fall back to global filesystem template
        tpl = get_global_template(pc)
        tpl_source = get_global_template_source(pc)

    undef = get_template_undef(tpl_source)
    parsed_content = pcTplEnv.parse(tpl_source)

    fill_env_values(kwargs, undef)

    errors = check_for_errors(undef, parsed_content, tpl_source, kwargs)
    if errors is not None:
        return errors

    pc_template = json.loads(tpl.render(**kwargs).replace("\n", ""))
    return pc_template["template"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

actinia-module viewer
Templates can be stored file based and in kvdb

* List all actinia-modules
* Describe single actinia-module
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2019-2025, mundialis"
__maintainer__ = "Carmen Tawalika"


from flask import jsonify, make_response
from flask_restful_swagger_2 import swagger
from flask_restful import Resource

from actinia_rest_lib.resource_base import ResourceBase

from actinia_module_plugin.apidocs import modules
from actinia_module_plugin.core.filter import filter
from actinia_module_plugin.core.modules.actinia_global_templates import (
    createProcessChainTemplateListFromFileSystem,
)
from actinia_module_plugin.core.modules.actinia_user_templates import (
    createProcessChainTemplateListFromKvdb,
)
from actinia_module_plugin.core.modules.actinia_common import (
    createActiniaModule,
)
from actinia_module_plugin.model.modules import ModuleList
from actinia_module_plugin.model.responseModels import (
    SimpleStatusCodeResponseModel,
)


class ListProcessChainTemplates(Resource):
    """List all process chain templates"""

    @swagger.doc(modules.listModules_get_docs)
    def get(self):
        """Get a list of all actinia modules (process chain templates)."""

        pc_list_fs = createProcessChainTemplateListFromFileSystem()
        pc_list_kvdb = createProcessChainTemplateListFromKvdb()
        pc_list = pc_list_fs + pc_list_kvdb

        pc_list = filter(pc_list)

        return make_response(
            jsonify(ModuleList(status="success", processes=pc_list)), 200
        )


class DescribeProcessChainTemplate(ResourceBase):
    """
    Describe process chain template as "virtual GRASS module"

    Contains HTTP GET endpoint
    Contains swagger documentation
    """

    @swagger.doc(modules.describeActiniaModule_get_docs)
    def get(self, actiniamodule):
        """Describe an actinia module (process chain template)."""

        try:
            virtual_module = createActiniaModule(self, actiniamodule)
            return make_response(jsonify(virtual_module), 200)
        except Exception:
            msg = 'Error looking for actinia module "' + actiniamodule + '".'
            res = jsonify(
                SimpleStatusCodeResponseModel(status=404, message=msg)
            )
            return make_response(res, 404)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

GRASS GIS module viewer
"""

__license__ = "Apache-2.0"
__author__ = "Anika Weinmann, Carmen Tawalika, Julia Haas"
__copyright__ = "Copyright 2019 - 2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


import json
import time

from actinia_core.models.response_models import create_response_from_model
from actinia_core.core.common.config import Configuration

from actinia_module_plugin.core.modules.processor import run_process_chain
from actinia_module_plugin.core.modules.parser import ParseInterfaceDescription
from actinia_module_plugin.model.modules import Module
from actinia_module_plugin.resources.logging import log
from actinia_module_plugin.core.modules.grass_modules_kvdb_interface import (
    kvdb_grass_module_interface,
)
from actinia_module_plugin.core.modules.accessible_modules_kvdb_interface import (
    getAccessibleModuleListKvdb,
)


def installGrassAddon(self, addon_name):
    """This function installs an official GRASS addon."""
    process_chain = {
        "version": 1,
        "list": [
            {
                "id": "1",
                "module": "g.extension",
                "inputs": [{"param": "extension", "value": addon_name}],
                "flags": "s",
            }
        ],
    }

    response = run_process_chain(self, process_chain)

    return response


def createModuleList(self):
    process_chain = {
        "version": 1,
        "list": [
            {
                "id": "g_search_modules_1",
                "module": "g.search.modules",
                "inputs": [{"param": "keyword", "value": ""}],
                "flags": "j",
            }
        ],
    }

    response = run_process_chain(self, process_chain)

    j_data = json.loads(response["process_log"][-1]["stdout"])

    # overwrite previous entries commited by EphemeralModuleLister in case the
    # further processing fails (e.g. invalid json). Else, the resource exists
    # and shows the output of g.search.modules.
    data = create_response_from_model(
        user_id=self.user_id,
        resource_id=self.resource_id,
        status="",
        orig_time=time.time(),
        orig_datetime="",
        message="",
    )
    self.resource_logger.commit(
        user_id=self.user_id,
        resource_id=self.resource_id,
        iteration=1,
        document=data,
        expiration=1,
    )

    module_list = []
    for data in j_data:
        description = data["attributes"]["description"]
        keywords = data["attributes"]["keywords"]
        name = data["name"]
        categories = keywords.split(",")
        categories.append("grass-module")
        module_response = Module(
            id=name, description=description, categories=sorted(categories)
        )
        module_list.append(module_response)

    return module_list


def createModuleUserList(self):
    return getAccessibleModuleListKvdb(self)


def build_and_run_iface_description_pc(self, module_list):
    pc = {"version": 1, "list": []}

    process_chain_items = []
    count = 1
    for module in module_list:
        if type(module) is str:
            module_id = module
        else:
            module_id = module["id"]
        process_chain_items.append(
            {
                "id": str(count),
                "module": module_id,
                "interface-description": True,
            }
        )
        count = count + 1

    pc["list"] = process_chain_items

    response = run_process_chain(self, pc)

    return response["process_log"]


def connect():
    """This method initializes the connection with kvdb."""
    conf = Configuration()
    try:
        conf.read()
    except Exception:
        pass

    server = conf.KVDB_SERVER_URL
    port = conf.KVDB_SERVER_PORT
    if conf.KVDB_SERVER_PW:
        kvdb_password = conf.KVDB_SERVER_PW
    else:
        kvdb_password = None

    kvdb_grass_module_interface.connect(
        host=server, port=port, password=kvdb_password
    )

    return kvdb_grass_module_interface


def cacheGrassModule(grass_module):
    """
    Insert grass_module into database
    """
    kvdb_grass_module_interface = connect()
    cached_module = kvdb_grass_module_interface.create(grass_module)

    return cached_module


def createGrassModule(self, module):
    module_list = [module]
    process_log = build_and_run_iface_description_pc(self, module_list)

    xml_string = process_log[0]["stdout"]
    grass_module = ParseInterfaceDescription(xml_string)

    # Check if cached module exists, then update if it has changed
    # This is currently the only mechanism to update a cached
    # module via API call.
    # If not, then cache it
    kvdb_grass_module_interface = connect()
    if kvdb_grass_module_interface.exists(module):
        kvdb_grass_module_interface.update(module, grass_module)
    else:
        cacheGrassModule(grass_module)

    return grass_module


def createFullModuleList(self, module_list):
    detailed_module_list = []
    request_module_list = []

    # Check if modules are already cached, then use them
    for module in module_list:
        kvdb_grass_module_interface = connect()
        grass_module = kvdb_grass_module_interface.read(module["id"])
        if grass_module:
            detailed_module_list.append(grass_module)
        else:
            request_module_list.append(module)

    # If they are not in cache, request and cache them
    if len(request_module_list) > 0:
        process_log = build_and_run_iface_description_pc(
            self, request_module_list
        )

        for desc in process_log:
            try:
                grass_module = ParseInterfaceDescription(desc["stdout"])
                cacheGrassModule(grass_module)
                detailed_module_list.append(grass_module)
            except Exception:
                log.error("error parsing module %s" % desc["executable"])

    return detailed_module_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Process chain template management
CRUD
* Create process chains template
* Read process chain template
* Update process chain template
* Delete process chain template
* List all process chains templates
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"
__maintainer__ = "Carmen Tawalika"


from flask import jsonify, make_response, request
from flask_restful_swagger_2 import swagger
from actinia_rest_lib.resource_base import ResourceBase

from actinia_module_plugin.apidocs import templates
from actinia_module_plugin.core.templates.user_templates import readAll
from actinia_module_plugin.core.templates.user_templates import createTemplate
from actinia_module_plugin.core.templates.user_templates import readTemplate
from actinia_module_plugin.core.templates.user_templates import updateTemplate
from actinia_module_plugin.core.templates.user_templates import deleteTemplate
from actinia_module_plugin.core.templates.global_templates import getAll
from actinia_module_plugin.core.templates.global_templates import getTemplate
from actinia_module_plugin.model.responseModels import (
    SimpleStatusCodeResponseModel,
)


class ActiniaTemplate(ResourceBase):
    """List all actinia templates (process chain templates)"""

    @swagger.doc(templates.listTemplates_get_docs)
    def get(self):
        """Get a list of all actinia templates (process chain templates)."""
        user_templates_list = readAll()
        global_templates_list = getAll()
        actinia_templates_list = user_templates_list + global_templates_list

        return make_response(jsonify(actinia_templates_list), 200)

    @swagger.doc(templates.createTemplate_post_docs)
    def post(self):
        """Create an actinia template (process chain template)."""
        template_id = request.get_json(force=True)["id"]
        actinia_template = createTemplate(request.get_json(force=True))
        if actinia_template is True:
            msg = 'Successfully created template "' + template_id + '".'
            status_code = 201
        elif actinia_template is False:
            msg = (
                'Error creating template "'
                + template_id
                + '", it already '
                + "exists. Please delete it before or use HTTP PUT to "
                + "update."
            )
            status_code = 400

        res = jsonify(
            SimpleStatusCodeResponseModel(status=status_code, message=msg)
        )
        return make_response(res, status_code)


class ActiniaTemplateId(ResourceBase):
    """Manage actinia templates (process chain templates)"""

    @swagger.doc(templates.readTemplate_get_docs)
    def get(self, template_id):
        """Read an actinia template (process chain template)."""

        msg = 'Error looking for actinia module "' + template_id + '".'
        res = jsonify(SimpleStatusCodeResponseModel(status=404, message=msg))

        try:
            actinia_template = readTemplate(template_id)
            if actinia_template is not False:
                return make_response(jsonify(actinia_template), 200)

            actinia_template = getTemplate(template_id)
            if actinia_template is not False:
                return make_response(jsonify(actinia_template), 200)
            else:
                return make_response(res, 404)
        except Exception:
            return make_response(res, 404)

    @swagger.doc(templates.updateTemplate_put_docs)
    def put(self, template_id):
        """Update an actinia template (process chain template)."""
        res = jsonify(
            SimpleStatusCodeResponseModel(status=404, message="Error")
        )
        try:
            actinia_template = updateTemplate(
                template_id, request.get_json(force=True)
            )
            if actinia_template is not False:
                return make_response(jsonify(actinia_template), 201)
            else:
                return make_response(res, 404)
        except Exception:
            return make_response(res, 404)

    @swagger.doc(templates.deleteTemplate_delete_docs)
    def delete(self, template_id):
        """Delete an actinia template (process chain template)."""
        res = jsonify(
            SimpleStatusCodeResponseModel(status=404, message="Error")
        )
        try:
            actinia_template = deleteTemplate(template_id)
            if actinia_template is True:
                return make_response(jsonify(actinia_template), 200)
            else:
                return make_response(res, 404)
        except Exception:
            return make_response(res, 404)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

actinia-module viewer
Module for file based templates which are global
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2019, mundialis"
__maintainer__ = "Carmen Tawalika"


import json

from actinia_module_plugin.core.common import filter_func
from actinia_module_plugin.model.modules import Module
from actinia_module_plugin.resources.templating import pcTplEnv
from actinia_module_plugin.resources.logging import log


def createProcessChainTemplateListFromFileSystem():
    """
    list all stored templates and return as actinia-module list
    """

    pc_list = []
    tpl_list = pcTplEnv.list_templates(filter_func=filter_func)

    for tpl_string in tpl_list:
        tpl = pcTplEnv.get_template(tpl_string)
        try:
            pc_template = json.loads(tpl.render().replace("\n", ""))
        except Exception:
            log.error("Error parsing template " + tpl_string)

        try:
            tpl_id = pc_template["id"]
            description = pc_template["description"]
            categories = ["actinia-module", "global-template"]

            pc_response = Module(
                id=tpl_id, description=description, categories=categories
            )
            pc_list.append(pc_response)

        except KeyError:
            log.warning("Could not read template %s" % pc_template)

    return pc_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2018-2021, mundialis"


import unittest

from actinia_module_plugin.resources.cli import pc2grass


class CliTest(unittest.TestCase):
    def test_cli_pc2grass(self):
        """Test basic cli command"""
        assert pc2grass() is None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Module management related to process chain templates
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2019-2025, mundialis"
__maintainer__ = "Carmen Tawalika"


def build_kwargs_for_template_rendering(module):
    """
    This method receives a process chain for an actinia module, isolates
    the received values and returns them so they can be filled into the
    process chain template.
    """
    kwargs = {}
    inOrOutputs = []

    if module.get("inputs") is not None:
        inOrOutputs += module.get("inputs")

    if module.get("outputs") is not None:
        inOrOutputs += module.get("outputs")

    for item in inOrOutputs:
        if (item.get("param") is None) or (item.get("value") is None):
            return None
        key = item["param"]
        val = item["value"]
        kwargs[key] = val

    return kwargs



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Process Chain Template Management
Kvdb CRUD for user templates
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"
__maintainer__ = "Carmen Tawalika"


from actinia_core.core.common.config import Configuration

from actinia_module_plugin.core.templates.user_templates_kvdb_interface import (
    kvdb_actinia_template_interface,
)


def connect():
    """This method initializes the connection with kvdb."""
    conf = Configuration()
    try:
        conf.read()
    except Exception:
        pass

    server = conf.KVDB_SERVER_URL
    port = conf.KVDB_SERVER_PORT
    if conf.KVDB_SERVER_PW:
        kvdb_password = conf.KVDB_SERVER_PW
    else:
        kvdb_password = None

    kvdb_actinia_template_interface.connect(
        host=server, port=port, password=kvdb_password
    )

    return kvdb_actinia_template_interface


def readAll():
    """
    Get all actinia templates from kvdb database
    """
    kvdb_actinia_template_interface = connect()
    actinia_template = kvdb_actinia_template_interface.list_all_ids()

    return actinia_template


def createTemplate(pc_tpl):
    """
    Insert actinia template into database
    """
    kvdb_actinia_template_interface = connect()
    actinia_template = kvdb_actinia_template_interface.create(pc_tpl)

    return actinia_template


def readTemplate(template_id):
    """
    Get actinia template by id
    """
    kvdb_actinia_template_interface = connect()
    actinia_template = kvdb_actinia_template_interface.read(template_id)

    return actinia_template


def updateTemplate(template_id, pc_tpl):
    """
    Update actinia template by id
    """
    kvdb_actinia_template_interface = connect()
    actinia_template = kvdb_actinia_template_interface.update(
        template_id, pc_tpl
    )

    return actinia_template


def deleteTemplate(template_id):
    """
    Delete actinia template by id
    """
    kvdb_actinia_template_interface = connect()
    actinia_template = kvdb_actinia_template_interface.delete(template_id)

    return actinia_template



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Dummy conftest.py for actinia_module_plugin.

If you don't know what this is for, just leave it empty.
Read more about conftest.py under:
https://pytest.org/latest/plugins.html
"""
from __future__ import print_function, absolute_import, division

# import pytest



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Template Environments
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-present mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


from jinja2 import Environment, PackageLoader, FileSystemLoader

from actinia_module_plugin.resources.config import PCTEMPLATECONFIG


# this environment is used for all cases where individual templates are loaded
tplEnv = Environment(
    loader=PackageLoader("actinia_module_plugin", "templates")
)

# this environment is used for process chain templates only
pcTplEnv = Environment(
    # loader=PackageLoader('actinia_module_plugin', 'templates/pc_templates')
    loader=FileSystemLoader(PCTEMPLATECONFIG.pathfile)
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

actinia-module viewer
Module for templates stored in kvdb which are user defined
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"
__maintainer__ = "Carmen Tawalika"


from actinia_module_plugin.core.templates.user_templates import readAll
from actinia_module_plugin.core.templates.user_templates import readTemplate
from actinia_module_plugin.model.modules import Module


def createProcessChainTemplateListFromKvdb():
    """
    list all stored templates and return as actinia-module list
    """

    pc_list = []
    tpl_list = readAll()

    for tpl_string in tpl_list:
        tpl = readTemplate(tpl_string)

        tpl_id = tpl["id"]
        description = tpl["description"]
        categories = ["actinia-module", "user-template"]

        pc_response = Module(
            id=tpl_id, description=description, categories=categories
        )
        pc_list.append(pc_response)

    return pc_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Common api methods
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2021 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


from flask_restful_swagger_2 import Schema


class SimpleStatusCodeResponseModel(Schema):
    """Simple response schema to inform about status."""

    type = "object"
    properties = {
        "status": {
            "type": "number",
            "description": "The status code of the request.",
        },
        "message": {
            "type": "string",
            "description": "A short message to describes the status",
        },
    }
    required = ["status", "message"]


simpleResponseExample = SimpleStatusCodeResponseModel(
    status=200, message="success"
)
SimpleStatusCodeResponseModel.example = simpleResponseExample



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2016-2025 Sören Gebbert & mundialis GmbH & Co. KG

SPDX-License-Identifier: GPL-3.0-or-later

Code based on actinia_core: https://github.com/actinia-org/actinia-core
actinia-core - an open source REST API for scalable, distributed, high
performance processing of geographical data that uses GRASS GIS for
computational tasks. For details, see https://actinia.mundialis.de/

Module viewer to run GRASS tasks
"""

__license__ = "GPLv3"
__author__ = "Anika Bettge"
__copyright__ = "Copyright 2016-2025, Sören Gebbert, mundialis GmbH & Co. KG"
__maintainer__ = "Anika Bettge"


import os
import shutil
import pickle
import uuid

from actinia_core.core.common.config import global_config
from actinia_processing_lib.ephemeral_processing import EphemeralProcessing
from actinia_core.models.response_models import (
    StringListProcessingResultResponseModel,
)

from actinia_module_plugin.core.common import start_job


def initGrass(self):
    """
    * not using enqueue_job to get always a response
    * the function creates a new project cause not all users can access
    a project
    """

    # check if project exists
    project_name = "project_for_listing_modules_" + str(uuid.uuid4())
    # '/actinia_core/grassdb/project_for_listing_modules'
    project = os.path.join(global_config.GRASS_DATABASE, project_name)
    # Check the project path
    if os.path.isdir(project):
        msg = (
            "Unable to create project. "
            "Project <%s> exists in global database." % project_name
        )
        return self.get_error_response(message=msg)
    # Check also for the user database
    # '/actinia_core/userdata/superadmin/project_for_listing_modules'
    project = os.path.join(
        self.grass_user_data_base, self.user_group, project_name
    )
    # Check the project path
    if os.path.isdir(project):
        msg = (
            "Unable to create project. "
            "Project <%s> exists in user database." % project_name
        )
        return self.get_error_response(message=msg)

    # create new project cause not each user can access a project
    if not os.path.isdir(
        os.path.join(self.grass_user_data_base, self.user_group)
    ):
        os.mkdir(os.path.join(self.grass_user_data_base, self.user_group))
    os.mkdir(project)
    mapset = os.path.join(project, "PERMANENT")
    os.mkdir(mapset)
    with open(os.path.join(mapset, "DEFAULT_WIND"), "w") as out:
        wind = (
            "proj:       3\nzone:       0\nnorth:      1N\n"
            + "south:      0\neast:       1E\nwest:       0\ncols:       1"
            + "\nrows:       1\ne-w resol:  1\nn-s resol:  1\ntop:        "
            + "1.000000000000000\nbottom:     0.000000000000000\ncols3:   "
            + "   1\nrows3:      1\ndepths:     1\ne-w resol3: 1\nn-s reso"
            + "l3: 1\nt-b resol:  1"
        )
        out.write(wind)
    with open(os.path.join(mapset, "MYNAME"), "w") as out:
        out.write("")
    with open(os.path.join(mapset, "PROJ_EPSG"), "w") as out:
        out.write("epsg: 4326")
    with open(os.path.join(mapset, "PROJ_INFO"), "w") as out:
        out.write(
            "name: WGS 84\ndatum: wgs84\nellps: wgs84\nproj: ll\n"
            + "no_defs: defined\ntowgs84: 0.000,0.000,0.000"
        )
    with open(os.path.join(mapset, "PROJ_UNITS"), "w") as out:
        out.write("unit: degree\nunits: degrees\nmeters: 1.0")
    with open(os.path.join(mapset, "WIND"), "w") as out:
        wind = (
            "proj:       3\nzone:       0\nnorth:      1N\n"
            + "south:      0\neast:       1E\nwest:       0\ncols:       1"
            + "\nrows:       1\ne-w resol:  1\nn-s resol:  1\ntop:        "
            + "1.000000000000000\nbottom:     0.000000000000000\ncols3:   "
            + "   1\nrows3:      1\ndepths:     1\ne-w resol3: 1\nn-s reso"
            + "l3: 1\nt-b resol:  1"
        )
        out.write(wind)

    return project_name


def deinitGrass(self, project_name):
    """
    * the function deletes above project
    """
    # remove project
    project = os.path.join(global_config.GRASS_DATABASE, project_name)
    if os.path.isdir(project):
        shutil.rmtree(project)
    project = os.path.join(
        self.grass_user_data_base, self.user_group, project_name
    )
    if os.path.isdir(project):
        shutil.rmtree(project)
    # del
    # self.user_credentials["permissions"]['accessible_datasets'][project_name]


class EphemeralModuleLister(EphemeralProcessing):
    """
    Overwrites EphemeralProcessing from actinia_core to bypass permission
    check for modules and temporary project, needed for self-description
    """

    def __init__(self, *args, pc):
        EphemeralProcessing.__init__(self, *args)
        self.response_model_class = StringListProcessingResultResponseModel
        self.process_chain = pc

    def _execute(self, skip_permission_check=True):
        self._setup()

        # Create the temporary database and link all available mapsets into it
        self._create_temp_database()

        process_list = self._validate_process_chain(
            process_chain=self.process_chain, skip_permission_check=True
        )

        self._create_grass_environment(
            grass_data_base=self.temp_grass_data_base, mapset_name="PERMANENT"
        )

        self._execute_process_list(process_list)

        self.module_results = self.module_output_log[0]["stdout"]


def run_process_chain(self, process_chain):
    """
    Used to list all GRASS modules, to describe a certain GRASS module
    and to generate actinia module description out of containing GRASS modules.
    ATTENTION! This call skips permission checks, so temporary project can be
    used. If user is not allowed to use GRASS modules used here, this will be
    allowed in these cases.
    """

    project_name = initGrass(self)

    # self.user_credentials["permissions"]['accessible_datasets'][project_name]
    # = ['PERMANENT']

    rdc = self.preprocess(
        has_json=False,
        has_xml=False,
        project_name=project_name,
        mapset_name="PERMANENT",
    )

    def list_modules(*args, process_chain=process_chain):
        processing = EphemeralModuleLister(*args, pc=process_chain)
        processing.run()

    if rdc:
        start_job(self.job_timeout, list_modules, rdc)
        _http_code, response_model = self.wait_until_finish()
    else:
        _http_code, response_model = pickle.loads(self.response_data)

    deinitGrass(self, project_name)

    return response_model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Filter for module and template viewer
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"
__maintainer__ = "Carmen Tawalika"


from flask import request


global family
global tag
global category


def filter(full_list):
    filter_list = full_list

    # INFO: record=full is directly parsed in api method

    if "family" in request.args:
        global family
        family = request.args["family"]
        # d, db, g, i, m, ps, r, r3, t, test, v
        val = family + "."
        filter_list = [i for i in filter_list if str(i["id"]).startswith(val)]

    if "tag" in request.args:
        global tag
        tag = request.args["tag"]
        filter_list = [i for i in filter_list if tag in str(i["categories"])]

    if "category" in request.args:
        global category
        category = request.args["category"]
        filter_list = [
            i for i in filter_list if category in str(i["categories"])
        ]

    return filter_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Accessible Modules Kvdb Interface
"""

__license__ = "Apache-2.0"
__author__ = "Anika Weinmann, Carmen Tawalika, Guido Riembauer, Julia Haas"
__copyright__ = "Copyright 2021 - 2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"

from actinia_core.core.kvdb_user import KvdbUserInterface
from actinia_core.core.common.config import Configuration


def getAccessibleModuleListKvdb(self):
    kvdb_interface = KvdbUserInterface()
    conf = Configuration()
    try:
        conf.read()
    except Exception:
        pass

    server = conf.KVDB_SERVER_URL
    port = conf.KVDB_SERVER_PORT
    if conf.KVDB_SERVER_PW:
        kvdb_password = conf.KVDB_SERVER_PW
    else:
        kvdb_password = None
    kvdb_interface.connect(host=server, port=port, password=kvdb_password)
    user = self.user.get_id()
    access_modules = kvdb_interface.get_credentials(user)["permissions"][
        "accessible_modules"
    ]
    kvdb_interface.disconnect()
    return access_modules


def addGrassAddonToModuleListKvdb(self, grassmodule):
    """
    This function adds installed GRASS addon to the user's module list
    in kvdb.
    """
    self.user.add_accessible_modules(
        [
            grassmodule,
        ]
    )
    self.user.update()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test Template CRUD
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"


from flask import Response
import json
import uuid

from actinia_api import URL_PREFIX

from testsuite import ActiniaTestCase


global allTemplatesCount
global templateUUID


class ActiniaTemplatesTest(ActiniaTestCase):
    def test_1001_read_all(self):
        """
        Test HTTP GET /actinia_templates and rememberes number of
        templates
        """
        global allTemplatesCount

        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/actinia_templates", headers=self.user_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        assert type(resp.json) is list
        allTemplatesCount = len(resp.json)

    def test_1002_create(self):
        """Test HTTP POST /actinia_templates (template creation)"""
        global templateUUID

        respStatusCode = 201

        with open("tests/resources/actinia_templates/pc_tpl.json") as file:
            pc_template = json.load(file)
        templateUUID = str(uuid.uuid4())
        pc_template["id"] = templateUUID

        resp = self.app.post(
            URL_PREFIX + "/actinia_templates",
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert "Success" in resp.json["message"]

    def test_1003_create_fail(self):
        """
        Test HTTP POST /actinia_templates (template creation) of already
        existing template
        """
        global templateUUID

        respStatusCode = 400

        with open("tests/resources/actinia_templates/pc_tpl.json") as file:
            pc_template = json.load(file)
        pc_template["id"] = templateUUID

        resp = self.app.post(
            URL_PREFIX + "/actinia_templates",
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

    def test_1004_read_single(self):
        """
        Test HTTP GET /actinia_templates/<template_id> of new created
        template
        """
        global templateUUID

        respStatusCode = 200

        resp = self.app.get(
            URL_PREFIX + "/actinia_templates/" + templateUUID,
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        with open("tests/resources/actinia_templates/pc_tpl.json") as file:
            pc_template = json.load(file)
        pc_template["id"] = templateUUID
        assert resp.json == pc_template

    def test_1005_update(self):
        """Test HTTP PUT /actinia_templates/<template_id> (update)"""
        global templateUUID

        respStatusCode = 201
        with open("tests/resources/actinia_templates/pc_tpl2.json") as file:
            pc_template = json.load(file)
        pc_template["id"] = templateUUID

        resp = self.app.put(
            URL_PREFIX + "/actinia_templates/" + templateUUID,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert resp.json is True

    def test_1006_read_update(self):
        """
        Test HTTP GET /actinia_templates/<template_id> of updated
        template
        """
        global templateUUID

        respStatusCode = 200

        resp = self.app.get(
            URL_PREFIX + "/actinia_templates/" + templateUUID,
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        with open("tests/resources/actinia_templates/pc_tpl2.json") as file:
            pc_template = json.load(file)
        pc_template["id"] = templateUUID
        assert resp.json == pc_template

    def test_1007_read_all_after_create(self):
        """
        Test HTTP GET /actinia_templates and compares number of templates
        from before (+1)
        """
        global allTemplatesCount

        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/actinia_templates", headers=self.user_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        assert type(resp.json) is list
        assert len(resp.json) == (allTemplatesCount + 1)

    def test_1008_delete(self):
        """Test HTTP DELETE /actinia_templates/<template_id> (deletion)"""
        global templateUUID

        respStatusCode = 200
        resp = self.app.delete(
            URL_PREFIX + "/actinia_templates/" + templateUUID,
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert resp.json is True

    def test_1009_read_all_after_delete(self):
        """
        Test HTTP GET /actinia_templates and compares number of templates
        from before (-1)
        """
        global allTemplatesCount

        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/actinia_templates", headers=self.user_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        assert type(resp.json) is list
        assert len(resp.json) == allTemplatesCount

    def test_1010_read_fail(self):
        """
        Test HTTP GET /actinia_templates/<template_id> of non-existing
        template
        """
        respStatusCode = 404
        resp = self.app.get(
            URL_PREFIX + "/actinia_templates/not_exist",
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

    def test_1011_update_fail(self):
        """
        Test HTTP PUT /actinia_templates/<template_id> (update) of
        non-existing template
        """
        global templateUUID
        respStatusCode = 404

        with open("tests/resources/actinia_templates/pc_tpl.json") as file:
            pc_template = json.load(file)
        pc_template["id"] = templateUUID

        resp = self.app.put(
            URL_PREFIX + "/actinia_templates/not_exist",
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

    def test_1012_delete_fail(self):
        """
        Test HTTP DELETE /actinia_templates/<template_id> (deletion) of
        non-existing template
        """
        respStatusCode = 404
        resp = self.app.delete(
            URL_PREFIX + "/actinia_templates/not_exist",
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

    def test_1013_read_global_template(self):
        """Test HTTP GET /actinia_templates/<template_id> of global template"""
        respStatusCode = 200
        # path = 'tests/resources/actinia_templates/user_default_value.json'

        resp = self.app.get(
            URL_PREFIX + "/actinia_templates/default_value",
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        # with open(path) as file:
        #     pc_template = json.load(file)
        # pc_template['id'] = 'default_value'
        # TODO: compare to file
        # assert resp.json[0]['template'] == pc_template['template']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Module viewer to parser GRASS xml response
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2019, mundialis"
__maintainer__ = "Carmen Tawalika"


import collections
import json
import xmltodict

from actinia_module_plugin.model.modules import Module
from actinia_module_plugin.model.modules import ModuleParameter
from actinia_module_plugin.model.modules import ModuleParameterSchema
from actinia_module_plugin.resources.logging import log
from actinia_module_plugin.resources.templating import tplEnv


def logstring(module_id, param, key):
    log.debug(module_id + " " + param + " has no key " + key)
    pass


def setParameterKey(module_id, parameter):
    try:
        key = parameter["@name"]
    except KeyError:
        key = None
        logstring(module_id, key, "name")
    except TypeError:
        logstring(module_id, key, "name")
        log.error("Yet unknown error occured")

    return key


def setVirtualParameterKey(module_id, parameter):
    try:
        key = module_id + "_" + parameter["@name"]
    except KeyError:
        key = None
        logstring(module_id, key, "name")

    return key


def setParameterDescription(module_id, key, parameter, kwargs):
    param_descr = ""
    label = False

    try:
        param_descr = parameter["label"] + ". "
        kwargs["description"] = param_descr
        label = True
    except KeyError:
        # logstring(module_id, key, "label")
        pass
    try:
        param_descr += parameter["description"] + ". "
        kwargs["description"] = param_descr
    except KeyError:
        if label:
            logstring(module_id, key, "description")
            pass
        log.warning(
            "Neither label nor description set for param %s"
            % parameter["@name"]
        )

    return kwargs


def setParameterName(key, kwargs):
    kwargs["name"] = key
    return kwargs


def setParameterOptional(parameter, kwargs):
    try:
        required = parameter["@required"]
        if required == "yes":
            kwargs["optional"] = False
        else:
            kwargs["optional"] = True
    except KeyError:
        required = "False"

    return kwargs


def setParameterDefault(parameter, kwargs):
    try:
        kwargs["default"] = parameter["default"]
        if parameter["default"] is None:
            kwargs["default"] = ""
    except KeyError:
        pass

    return kwargs


def setParamType(module_id, key, parameter, schema_kwargs):
    try:
        # grass parameter types can only be string, double or integer
        gtype = parameter["@type"]
        if gtype in ("float", "double"):
            gtype = "number"
        schema_kwargs["type"] = gtype
    except KeyError:
        logstring(module_id, key, "type")
    try:
        multiple = parameter["@multiple"]
        if multiple == "yes":
            gtype = "array"
        schema_kwargs["type"] = gtype
    except KeyError:
        logstring(module_id, key, "multiple")
    try:
        for subtype_key, val in parameter["gisprompt"].items():
            if subtype_key == "@element":
                schema_kwargs["subtype"] = val
    except Exception:
        pass

    return schema_kwargs


def setParameterEnum(parameter, schema_kwargs):
    try:
        enum = []
        for enum_key, val in parameter["values"].items():
            for item in val:
                for i in item:
                    if i == "name":
                        enum.append(item[i])
        if len(enum) > 0:
            schema_kwargs["enum"] = enum
    except KeyError:
        pass

    return schema_kwargs


def isOutput(parameter):
    """
    Checks if parameter is output parameter.
    Returns True if parameter has key
    'gisprompt.age' == 'new',
    False otherwise.
    """
    try:
        if "@age" in parameter["gisprompt"].keys():
            return parameter["gisprompt"]["@age"] == "new"
        else:
            return False
    except KeyError:
        return False
    except AttributeError:
        return False


def createModuleParameterFromGrassParam(module_id, key, parameter):
    kwargs = dict()
    schema_kwargs = dict()

    schema_kwargs = setParamType(module_id, key, parameter, schema_kwargs)
    kwargs = setParameterDescription(module_id, key, parameter, kwargs)
    kwargs = setParameterName(key, kwargs)
    kwargs = setParameterOptional(parameter, kwargs)
    kwargs = setParameterDefault(parameter, kwargs)
    schema_kwargs = setParameterEnum(parameter, schema_kwargs)

    param_object = ModuleParameter(
        **kwargs, schema=ModuleParameterSchema(**schema_kwargs)
    )

    del kwargs
    del schema_kwargs

    return param_object


def createModuleParameterFromGrassFlag(module_id, parameter):
    kwargs = dict()
    kwargs["default"] = "False"
    kwargs["optional"] = True
    schema_kwargs = dict()
    schema_kwargs["type"] = "boolean"

    key = setParameterKey(module_id, parameter)

    kwargs = setParameterDescription(module_id, key, parameter, kwargs)
    kwargs = setParameterName(key, kwargs)

    param_object = ModuleParameter(
        **kwargs, schema=ModuleParameterSchema(**schema_kwargs)
    )
    del kwargs
    del schema_kwargs

    return param_object


def ParseInterfaceDescription(xml_string, keys=None):
    """
    Parses output of GRASS interface-description
    and returns openEO process object
    """

    gm_dict = xmltodict.parse(xml_string)["task"]

    module_id = gm_dict["@name"]
    description = gm_dict["description"]
    categories = gm_dict["keywords"].replace(" ", "").split(",")
    categories.append("grass-module")
    parameters = []
    returns = []
    extrakwargs = dict()

    # if a GRASS GIS module has only one parameter, xml2dict does not transform
    # parameters to an array but only assigns this one parameter. For our
    # parser to get along, we transform in this case to array as well.
    if "parameter" not in xmltodict.parse(xml_string)["task"]:
        gm_dict["parameter"] = []
    elif (
        type(xmltodict.parse(xml_string)["task"]["parameter"])
        == collections.OrderedDict
        or type(xmltodict.parse(xml_string)["task"]["parameter"]) == dict
    ):
        gm_dict["parameter"] = [gm_dict["parameter"]]
    else:
        # keep as is
        pass

    try:
        grass_params = gm_dict["parameter"]
    except KeyError:
        logstring(module_id, "", "has no parameter")
        grass_params = []

    try:
        flags = gm_dict["flag"]
    except KeyError:
        logstring(module_id, "", "has no flags")
        flags = []

    for parameter in grass_params:
        if keys:
            # case for actinia modules
            key = setVirtualParameterKey(module_id, parameter)
            key_exists = False
            if key not in keys:
                for actiniakey in keys:
                    if actiniakey.startswith(key):
                        key_exists = True
            else:
                key_exists = True
            if not key_exists:
                continue
        else:
            # case for GRASS modules
            key = setParameterKey(module_id, parameter)
        param_object = createModuleParameterFromGrassParam(
            module_id, key, parameter
        )
        if isOutput(parameter):
            returns.append(param_object)
        else:
            parameters.append(param_object)

    for parameter in flags:
        # not possible to specify flag values via template at the moment
        if keys:
            continue
        param_object = createModuleParameterFromGrassFlag(module_id, parameter)

        parameters.append(param_object)

    # custom extention for importer + exporter from actinia_core
    # Runs when viewing importer/exporter as module and when used in template.
    try:
        tpl = tplEnv.get_template("gmodules/" + module_id + ".json")
        pc_template = json.loads(tpl.render().replace("\n", ""))
        for key in [*pc_template]:
            extrakwargs[key] = []
            for param in pc_template[key]:
                extrakwargs[key].append(ModuleParameter(**param))
    except Exception:
        # if no template for module exist, use as is (default)
        # log.debug('template %s does not exist.', e)
        pass

    grass_module = Module(
        id=module_id,
        description=description,
        categories=sorted(categories),
        parameters=parameters,
        returns=returns,
        **extrakwargs,
    )

    return grass_module



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test Module Lists and Self-Description
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"


# import unittest

from flask import Response

from actinia_api import URL_PREFIX

from actinia_core.version import init_versions, G_VERSION

from testsuite import ActiniaTestCase, compare_module_to_file


someGrassModules = ["r.slope.aspect", "importer", "exporter"]


class GmodulesTest(ActiniaTestCase):

    # For expected test results, which are dependent on GRASS GIS version
    init_versions()
    grass_version_s = G_VERSION["version"]
    grass_version = [int(item) for item in grass_version_s.split(".")[:2]]

    # @unittest.skip("demonstrating skipping")
    def test_list_modules_get_user(self):
        """Test HTTP GET /grass_modules for user"""
        global someGrassModules

        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/grass_modules", headers=self.user_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert len(resp.json["processes"]) > 150
        assert "categories" in resp.json["processes"][0]
        assert "description" in resp.json["processes"][0]
        assert "id" in resp.json["processes"][0]

        respModules = [i["id"] for i in resp.json["processes"]]
        for i in someGrassModules:
            assert i in respModules

    def test_list_modules_get_restricted_user(self):
        """Test HTTP GET /grass_modules for restricted user"""
        global someGrassModules

        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/grass_modules",
            headers=self.restricted_user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert len(resp.json["processes"]) == 3
        assert "categories" in resp.json["processes"][0]
        assert "description" in resp.json["processes"][0]
        assert "id" in resp.json["processes"][0]

    def test_filter_list_modules_get_user(self):
        """Test HTTP GET /grass_modules with filter as user"""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/grass_modules?category=slope",
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        # only r.slope.aspect is permitted
        assert len(resp.json["processes"]) == 1

    def test_filter_list_modules_get_admin(self):
        """Test HTTP GET /grass_modules with filter as admin"""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/grass_modules?category=slope",
            headers=self.admin_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        # both r.slope.aspect and v.to.db should be allowed
        assert len(resp.json["processes"]) == 2

    def test_filter_list_modules_get_admin_2(self):
        """Test HTTP GET /grass_modules with filter as admin"""
        respStatusCode = 200
        url_path = "/grass_modules?record=full&family=ps"
        resp = self.app.get(
            URL_PREFIX + url_path, headers=self.admin_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        assert len(resp.json["processes"]) == 1
        assert resp.json["processes"][0]["categories"] != 0
        assert resp.json["processes"][0]["parameters"] != 0

    def test_filter_list_modules_get_admin_3(self):
        """Test HTTP GET /grass_modules with filter"""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/grass_modules?family=test",
            headers=self.admin_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        if self.grass_version < [8, 5]:
            assert len(resp.json["processes"]) == 2
        else:
            assert len(resp.json["processes"]) == 3
        assert resp.json["processes"][0]["categories"] != 0
        assert hasattr(resp.json["processes"][0], "parameters") is False


for i in someGrassModules:
    """
    Test HTTP GET /grass_modules/<module> for GRASS GIS modules in loop
    for all examples in someGrassModules above and compares response to file
    """
    # create method for every grass-module to have a better overview in
    # test summary
    def_name = "test_describe_module_get_" + i
    compare_module_to_file.__defaults__ = (
        "grass_modules",
        i,
    )
    setattr(GmodulesTest, def_name, compare_module_to_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Application entrypoint. Creates Flask app and swagger docs, adds endpoints
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2021 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


from flask import Flask
from flask_cors import CORS
from flask_restful_swagger_2 import Api

from actinia_module_plugin import endpoints
from actinia_module_plugin.resources.logging import log


app = Flask(__name__)
CORS(app)

apidoc = Api(
    app,
    title="actinia-module-plugin",
    api_spec_url="/latest/api/swagger",
    schemes=["https", "http"],
    consumes=["application/json"],
    description="""Contains module self-description and process-chain-template
                   management and processing.
                   """,
)

endpoints.addEndpoints(app, apidoc)


if __name__ == "__main__":
    # call this for development only with
    # `python -m actinia_module_plugin.main`
    log.debug("starting app in development mode...")
    app.run(debug=True, use_reloader=False)
    # for production environent use application in wsgy.py



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Functions for process chain template parameters
"""

__author__ = "Carmen Tawalika, Anika Weinmann"
__copyright__ = "2018-2023 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


from jinja2 import meta, nodes

from re import findall as re_findall

from actinia_module_plugin.resources.templating import pcTplEnv


def find_filters(ast):
    """
    Find all the nodes of a given type. If the type is a tuple,
    the check is performed for any of the tuple items.
    Function from: https://stackoverflow.com/questions/55275399/how-to-get
    -variables-along-with-their-filter-name-from-jinja2-template
    """
    for child in ast.iter_child_nodes():
        if isinstance(child, nodes.Filter):
            yield child
        else:
            for result in find_filters(child):
                yield result


def filtered_variables(ast):
    """
    Return variables that have filters, along with their filters. Might
    return duplicate variable names with different filters
    Function from: https://stackoverflow.com/questions/55275399/how-to-get
    -variables-along-with-their-filter-name-from-jinja2-template
    """
    results = []
    for i, node in enumerate(find_filters(ast)):
        filters = []
        f = node
        filters.append(f.name)
        while isinstance(f.node, nodes.Filter):
            f = f.node
            filters.append(f.name)
        filters.reverse()
        results.append((f.node.name, filters))
    return results


def get_template_default_values(parsed_content):
    """Function to find default variables from processchain"""
    default_vars = []
    filtered_vars = filtered_variables(parsed_content)
    for filtered_var in filtered_vars:
        if "default" in filtered_var[1]:
            default_vars.append(filtered_var[0])
    return default_vars


def filter_undef_without_if_statements(tpl_source, undef, not_needed_vars=[]):
    """
    Function to find variables which are only in an if statement and has
    not to be set
    """
    regex_m_all = []
    if "{% if" in tpl_source or "{%- if" in tpl_source:
        for i in undef:
            if i not in not_needed_vars:
                # be careful with { and {{, if you split the line differently
                # here because of linting
                r_str = (
                    rf"{{%.* if {i} is defined .*%}}[\S\n\t\v ]+?"
                    r"{%.* endif .*%}"
                )
                regex_m = re_findall(r_str, tpl_source)
                regex_m_all.extend(regex_m)

        if len(regex_m_all) > 0:
            tpl_source_mod = tpl_source
            for if_part in regex_m_all:
                tpl_source_mod = tpl_source_mod.replace(if_part, "")
            # check if the variable is still a variable in modified template
            undef_mod = get_template_undef(tpl_source_mod)
            for var in undef:
                if var not in not_needed_vars and var not in undef_mod:
                    not_needed_vars.append(var)


def get_not_needed_params(undef, tpl_source, parsed_content=None):
    """
    Function which filters undef parameters to find which are not needed
    because they have a default value or are only inside a if statement
    """

    if parsed_content is None:
        parsed_content = pcTplEnv.parse(tpl_source)

    # find default variables from processchain
    not_needed_vars = get_template_default_values(parsed_content)
    default_len = len(not_needed_vars)

    # find variables which are only in an if statement and has not to be set
    filter_undef_without_if_statements(tpl_source, undef, not_needed_vars)
    if_len = len(not_needed_vars) - default_len

    # return also the reason why the parameters is not needed
    reason = ["default"] * default_len + ["if"] * if_len

    return {key: val for key, val in zip(not_needed_vars, reason)}


def get_template_undef(tpl_source):
    parsed_content = pcTplEnv.parse(tpl_source)
    undef = meta.find_undeclared_variables(parsed_content)
    return undef



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

GRASS GIS module cache in kvdb
Kvdb interface
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"
__maintainer__ = "Carmen Tawalika"


import pickle

from actinia_core.core.common.kvdb_base import KvdbBaseInterface


class KvdbActiniaGrassModuleInterface(KvdbBaseInterface):
    """
    The Kvdb GRASS GIS module database interface

    A single GRASS GIS module is stored as Hash with:
        - GRASS GIS module name that must be unique
        - GRASS GIS module dictionary

    In addition is the GRASS GIS module name saved in a hash that contains all
    GRASS GIS module names.
    """

    grass_module_id_hash_prefix = "GRASS-MODULE-ID-HASH-PREFIX::"
    grass_module_id_db = "GRASS-MODULE-ID-DATABASE"

    def __init__(self):
        KvdbBaseInterface.__init__(self)

    def create(self, grass_module):
        """
        Add an grass_module to the grass_module database

        Args:
            grass_module (dict): A GRASS GIS interface description

        Returns:
            bool:
            True is success, False if grass_module is already in database
        """

        grass_module_id = grass_module["id"]

        keyname = self.grass_module_id_hash_prefix + grass_module_id
        exists = self.kvdb_server.exists(keyname)
        if exists == 1 or exists is True:
            return False

        grass_module_bytes = pickle.dumps(grass_module)
        mapping = {
            "grass_module_id": grass_module_id,
            "grass_module": grass_module_bytes,
        }

        lock = self.kvdb_server.lock(name="add_grass_module_lock", timeout=1)
        lock.acquire()
        # First add the grass_module-id to the grass_module id database
        self.kvdb_server.hset(
            self.grass_module_id_db, grass_module_id, grass_module_id
        )

        self.kvdb_server.hset(
            self.grass_module_id_hash_prefix + grass_module_id, mapping=mapping
        )
        lock.release()

        return True

    def read(self, grass_module_id):
        """
        Return the grass_module

        HGET grass_module-id grass_module_group

        Args:
            grass_module_id: The grass_module id

        Returns:
             str:
             The grass_module group
        """

        try:
            grass_module = pickle.loads(
                self.kvdb_server.hget(
                    self.grass_module_id_hash_prefix + grass_module_id,
                    "grass_module",
                )
            )
        except Exception:
            return False

        return grass_module

    def update(self, grass_module_id, grass_module):
        """
        Update the grass_module.

        Renaming an entry is not allowed, only existing entries with the
        same grass_module_id can be updated.

        Args:
            grass_module_id (str): The grass_module id
            grass_module (dict): The grass_module as dictionary

        Returns:
            bool:
            True is success, False if grass_module is not in the database

        """
        keyname = self.grass_module_id_hash_prefix + grass_module_id
        exists = self.kvdb_server.exists(keyname)
        if exists == 0 or exists is False:
            return False

        grass_module_bytes = pickle.dumps(grass_module)
        mapping = {
            "grass_module_id": grass_module_id,
            "grass_module": grass_module_bytes,
        }

        lock = self.kvdb_server.lock(
            name="update_grass_module_lock", timeout=1
        )
        lock.acquire()

        self.kvdb_server.hset(
            self.grass_module_id_hash_prefix + grass_module_id, mapping=mapping
        )

        lock.release()

        return True

    # def delete(self, grass_module_id):
    #     """
    #     Remove an grass_module id from the database

    #     Args:
    #         grass_module_id (str): The grass_module id

    #     Returns:
    #         bool:
    #         True is grass_module exists, False otherwise
    #     """
    #     exists = self.exists(grass_module_id)
    #     if exists == 0 or exists is False:
    #         return False

    #     lock = self.kvdb_server.lock(
    #         name="delete_grass_module_lock", timeout=1)
    #     lock.acquire()
    #     # Delete the entry from the grass_module id database
    #     self.kvdb_server.hdel(self.grass_module_id_db,
    #                            grass_module_id)
    #     # Delete the actual grass_module entry
    #     self.kvdb_server.delete(
    #         self.grass_module_id_hash_prefix + grass_module_id)
    #     lock.release()

    #     return True

    # def list_all_ids(self):
    #     """
    #     List all grass_module id's that are in the database

    #     HKEYS on the grass_module id database

    #     Returns:
    #         list:
    #         A list of all grass_module ids in the database
    #     """
    #     values = []
    #     list = self.kvdb_server.hkeys(self.grass_module_id_db)
    #     for entry in list:
    #         entry = entry.decode()
    #         values.append(entry)

    #     return values

    def exists(self, grass_module_id):
        """
        Check if the grass_module is in the database

        Args:
            grass_module_id (str): The grass_module id

        Returns:
            bool:
            True is grass_module exists, False otherwise
        """
        return self.kvdb_server.exists(
            self.grass_module_id_hash_prefix + grass_module_id
        )


# Create the Kvdb interface instance
kvdb_grass_module_interface = KvdbActiniaGrassModuleInterface()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Documentation objects for GRASS modules and actinia modules api endpoints
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2021 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


import copy

from actinia_module_plugin.model.responseModels import (
    SimpleStatusCodeResponseModel,
)

from actinia_module_plugin.model.modules import Module, ModuleList


null = "null"


listModules_get_docs = {
    "tags": ["Module Viewer"],
    "description": "Get a list of modules. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "in": "path",
            "name": "tag",
            "type": "string",
            "description": "Filter for categories",
        },
        {
            "in": "path",
            "name": "category",
            "type": "string",
            "description": "Another filter for categories",
        },
        {
            "in": "path",
            "name": "family",
            "type": "string",
            "description": "Type of GRASS GIS module",
            "enum": [
                "d",
                "db",
                "g",
                "i",
                "m",
                "ps",
                "r",
                "r3",
                "t",
                "test",
                "v",
            ],
        },
        {
            "in": "path",
            "name": "record",
            "type": "string",
            "description": "If set to 'full', all information about the "
            "returned modules are given like in the single "
            "module description. Depending on active cache, "
            "this response might run into a timeout. A filter "
            "can prevent this.",
        },
    ],
    "responses": {
        "200": {
            "description": "This response returns a list of module names and "
            "the status.",
            "schema": ModuleList,
        },
        "400": {
            "description": "The error message and a detailed log why listing "
            "of modules did not succeeded",
            "schema": SimpleStatusCodeResponseModel,
        },
    },
}

describeModule_get_docs = {
    "tags": ["Module Viewer"],
    "parameters": [
        {
            "in": "path",
            "name": "module",
            "type": "string",
            "description": "The name of a module",
            "required": True,
        }
    ],
    "description": "Get the description of a module. "
    "Minimum required user role: user."
    "Can be also used to reload cache for a certain module"
    "for the full module description in listModules.",
    "responses": {
        "200": {
            "description": "This response returns a description of a module.",
            "schema": Module,
        },
        "400": {
            "description": "The error message and a detailed log why "
            "describing modules did not succeeded",
            "schema": SimpleStatusCodeResponseModel,
        },
    },
}


describeActiniaModule_get_docs = copy.deepcopy(describeModule_get_docs)
describeActiniaModule_get_docs["parameters"][0]["name"] = "actiniamodule"

describeGrassModule_get_docs = copy.deepcopy(describeModule_get_docs)
describeGrassModule_get_docs["parameters"][0]["name"] = "grassmodule"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test processing
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"


from flask import Response
import json
from time import sleep

from actinia_api import URL_PREFIX


from testsuite import ActiniaTestCase, check_started_process


global allTemplatesCount
global templateUUID


class ActiniaProcessingTest(ActiniaTestCase):
    def test_processing(self):
        """Test Usage of global templates persistent processing"""

        respStatusCode = 200
        json_path = "tests/resources/processing/global_default_value.json"
        url_path = (
            f"/{self.project_url_part}/nc_spm_08/mapsets/test/processing"
        )

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

        resp = self.app.delete(
            f"{URL_PREFIX}/{self.project_url_part}/nc_spm_08/mapsets/test",
            headers=self.user_auth_header,
        )

    def test_processing_export(self):
        """Test Usage of global templates ephemeral processing"""
        respStatusCode = 200
        json_path = "tests/resources/processing/global_point_in_polygon.json"
        url_path = f"/{self.project_url_part}/nc_spm_08/processing_export"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

    def test_processing_if_statement_1(self):
        """
        Test Usage of global templates ephemeral processing with if
        statement where all variables are set"""
        respStatusCode = 200
        json_path = (
            "tests/resources/processing/global_if_statement_filled_all.json"
        )
        url_path = f"/{self.project_url_part}/nc_spm_08/processing_export"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

        # poll status until finished
        status = resp.json["status"]
        while status not in ["error", "finished"]:
            sleep(3)
            resp = self.app.get(
                resp.json["urls"]["status"], headers=self.user_auth_header
            )
            status = resp.json["status"]

        # check if parameter is set
        resp.json["process_chain_list"]
        params = [
            param["param"]
            for param in resp.json["process_chain_list"][0]["list"][0][
                "inputs"
            ]
        ]
        assert "region" in params, "Parameter 'region' is not set"

    def test_processing_if_statement_2(self):
        """
        Test Usage of global templates ephemeral processing with if
        statement where the variable in the if statement is not set"""
        respStatusCode = 200
        json_path = (
            "tests/resources/processing/global_if_statement_not_all.json"
        )
        url_path = f"/{self.project_url_part}/nc_spm_08/processing_export"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

        # poll status until finished
        status = resp.json["status"]
        while status not in ["error", "finished"]:
            sleep(3)
            resp = self.app.get(
                resp.json["urls"]["status"], headers=self.user_auth_header
            )
            status = resp.json["status"]

        # check if parameter is set
        resp.json["process_chain_list"]
        params = [
            param["param"]
            for param in resp.json["process_chain_list"][0]["list"][0][
                "inputs"
            ]
        ]
        assert "region" not in params, "Parameter 'region' is set"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test Module Lists and Self-Description
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2025, mundialis"

import json
from flask import Response

from actinia_api import URL_PREFIX

from testsuite import (
    ActiniaTestCase,
    check_started_process,
    delete_user_template,
    import_user_template,
)


class ActiniaModulesProcessingTest(ActiniaTestCase):
    def test_process_global_module(self):
        """
        Test HTTP POST /actinia_modules/<module>/processing of
        global templates ephemeral processing
        """
        respStatusCode = 200
        json_path = "tests/resources/processing/global_point_in_polygon.json"
        url_path = "/actinia_modules/point_in_polygon/process"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

    def test_process_user_module(self):
        """
        Test HTTP POST /actinia_modules/<module>/processing of
        user templates ephemeral processing
        """
        import_user_template(self, "user_point_in_polygon")

        respStatusCode = 200
        json_path = (
            "tests/resources/processing/"
            "user_point_in_polygon_actinia_module_process.json"
        )
        url_path = "/actinia_modules/user_point_in_polygon/process"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

        delete_user_template(self, "user_point_in_polygon")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Documentation objects for process chain template management
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2021 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


import os
import json

from flask_restful_swagger_2 import Schema
from actinia_core.core.common.process_chain import GrassModule

from actinia_module_plugin.model.responseModels import (
    SimpleStatusCodeResponseModel,
)


script_dir = os.path.dirname(os.path.abspath(__file__))
rel_path = "./examples/pc_template.json"
abs_file_path = os.path.join(script_dir, rel_path)
with open(abs_file_path) as jsonfile:
    pc_template_example = json.load(jsonfile)


null = "null"


class ProcessChainTemplate(Schema):
    """Response schema for module"""

    type = "object"
    properties = {
        "id": {
            "type": "string",
            "description": "Unique identifier of the process. ",
        },
        "description": {
            "type": "string",
            "description": "Detailed description to fully explain the entity.",
        },
        "template": {
            "type": "object",
            "description": "The full process chain template.",
            "properties": {
                "list": {
                    "type": "array",
                    "items": GrassModule,
                    "description": "The list of GRASS GIS or actinia modules"
                    " or executables of which the template"
                    " consists.",
                }
            },
        },
    }
    example = pc_template_example


listTemplates_get_docs = {
    "tags": ["Process Chain Template Management"],
    "description": "Get a list of process chain templates. "
    "Minimum required user role: user.",
    "responses": {
        "200": {"description": "This response returns a list of module names"}
    },
}

readTemplate_get_docs = {
    "tags": ["Process Chain Template Management"],
    "parameters": [
        {
            "in": "path",
            "name": "template_id",
            "type": "string",
            "description": "The name of a process chain template",
            "required": True,
        }
    ],
    "description": "Read a process chain template. "
    "Minimum required user role: user.",
    "responses": {
        "200": {
            "description": "This response returns a process chain template.",
            "schema": ProcessChainTemplate,
        },
        "404": {
            "description": "The error message and a detailed log why "
            "describing did not succeeded",
            "schema": SimpleStatusCodeResponseModel,
        },
    },
}


createTemplate_post_docs = {
    "tags": ["Process Chain Template Management"],
    "description": "Create a process chain template. "
    "Minimum required user role: user.",
    "parameters": [
        {
            "in": "body",
            "name": "template",
            "type": "object",
            "schema": ProcessChainTemplate,
            "description": "The process chain template",
            "required": True,
        }
    ],
    "responses": {
        "201": {
            "description": "This response returns True if creation was"
            " successfull."
        },
        "404": {
            "description": "The error message and a detailed log why "
            "creation did not succeeded",
            "schema": SimpleStatusCodeResponseModel,
        },
    },
}

updateTemplate_put_docs = {
    "tags": ["Process Chain Template Management"],
    "parameters": [
        {
            "in": "path",
            "name": "template_id",
            "type": "string",
            "description": "The name of a process chain template",
            "required": True,
        }
    ],
    "description": "Update a process chain template. "
    "Minimum required user role: user.",
    "responses": {
        "201": {
            "description": "This response returns True if update was"
            " successfull."
        },
        "404": {
            "description": "The error message and a detailed log why "
            "update did not succeeded",
            "schema": SimpleStatusCodeResponseModel,
        },
    },
}

deleteTemplate_delete_docs = {
    "tags": ["Process Chain Template Management"],
    "parameters": [
        {
            "in": "path",
            "name": "template_id",
            "type": "string",
            "description": "The name of a process chain template",
            "required": True,
        }
    ],
    "description": "Delete a process chain template. "
    "Minimum required user role: user.",
    "responses": {
        "200": {
            "description": "This response returns True if deletion was"
            " successfull."
        },
        "404": {
            "description": "The error message and a detailed log why "
            "deletion did not succeeded",
            "schema": SimpleStatusCodeResponseModel,
        },
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2018-2021, mundialis"


import unittest
import importlib
from flask import Response

import actinia_module_plugin
from testsuite import ActiniaTestCase, URL_PREFIX


class AppTest(ActiniaTestCase):
    def test_app_running(self):
        """Test if app responds"""
        resp = self.app.get(f"{URL_PREFIX}/version")
        assert isinstance(resp, Response)

    # see endpoints.py why this is outcommented
    # def test_app_responding(self):
    #     respStatusCode = 200
    #     resp = self.app.get('/')
    #     assert resp.status_code == respStatusCode

    def test_app_static(self):
        """Test if index cannot be found (see above)"""
        respStatusCode = 404
        resp = self.app.get("/index.html")
        assert resp.status_code == respStatusCode


class InitTest(unittest.TestCase):
    def test_init(self):
        """Test if version can be detected"""
        version = importlib.metadata.version("actinia_module_plugin")
        assert actinia_module_plugin.__version__ == version



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Commandline functions, extend later if needed
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-present mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


import os
import json
import sys


# used in pc2grass
def parseExe(process):
    # no api docs model
    line = process["exe"]

    if "params" in process:
        for param in process["params"]:
            line += " "
            line += param

    return line


# used in pc2grass
def parseModule(process):
    # id: string
    # module: string
    # inputs:  InputParameter
    # outputs:  OutputParameter
    # flags: string
    # stdin: string
    # overwrite: boolean
    # verbose: boolean
    # superquiet: boolean

    line = process["module"]

    if "inputs" in process:
        # param: string
        # value: string
        # import_descr: object
        for input in process["inputs"]:
            line += " "
            line += input["param"]
            line += "="
            line += '"' + input["value"] + '"'

        if "import_descr" in input:
            print(
                "WARNING: Cannot translate import_descr for " + process["id"]
            )

    if "outputs" in process:
        # param: string
        # value: string
        # export: object
        for output in process["outputs"]:
            line += " "
            line += input["param"]
            line += "="
            line += input["value"]

        if "export" in output:
            print("WARNING: Cannot translate export for " + process["id"])

    if "flags" in process:
        line += " -" + process["flags"]
    if "stdin" in process:
        print("WARNING: Cannot translate export for " + process["id"])
    if "overwrite" in process and process["overwrite"] == "true":
        line += " --overwrite"
    if "verbose" in process and process["verbose"] == "true":
        line += " --verbose"
    if "superquiet" in process and process["superquiet"] == "true":
        line += " --quiet"

    return line


def defineFile(file):
    if os.path.isfile(file):
        file = file
    elif os.path.isfile(os.getcwd() + file):
        file = os.getcwd() + file
    else:
        print("WARNING: Could not find file " + str(file))
        return
    return file


def pc2grass():
    """Parser for actinia-core process chains to GRASS executables"""

    try:
        input = sys.argv[1]
        output = sys.argv[2]
    except IndexError:
        print("ERROR: No input or output file given")
        return

    if os.path.isfile(input):
        input = input
        output = output
    elif os.path.isfile(os.getcwd() + input):
        input = os.getcwd() + input
        output = os.getcwd() + output
    else:
        print("ERROR: Could not find input file " + str(input))
        return

    with open(input, "r") as file:
        pc = json.loads(file.read())

    list = pc["list"]
    script = ""

    for process in list:
        line = ""
        if "exe" in process:
            line = parseExe(process)
        elif "module" in process:
            line = parseModule(process)
        else:
            print(
                "WARNING: Neither 'exe' nor 'module' found in " + process["id"]
            )

        script += line + "\n"

    try:
        with open(output, "x") as file:
            file.write(script)
    except FileExistsError:
        print(
            "ERROR: output file already exists."
            + " (No overwrite option yet, sorry)"
        )
        return

    print("All written")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test Module Lists and Self-Description
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"


from flask import Response

from actinia_api import URL_PREFIX

from testsuite import ActiniaTestCase, compare_module_to_file

GrassModules = ["r.slope.aspect", "importer", "exporter"]
someActiniaModules = [
    "add_enumeration",
    "default_value",
    "nested_modules_test",
    "point_in_polygon",
    "slope_aspect",
    "vector_area",
    "index_NDVI",
    "loop_simple",
    "loop",
]
someVirtualModules = GrassModules + someActiniaModules


class VirtualModulesTest(ActiniaTestCase):
    def test_list_virtual_modules_get(self):
        """Test HTTP GET /modules"""
        global someVirtualModules

        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules", headers=self.user_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert "grass-module" in resp.json["processes"][0]["categories"]
        assert "actinia-module" in resp.json["processes"][-1]["categories"]

        assert len(resp.json["processes"]) > 150
        assert "categories" in resp.json["processes"][0]
        assert "description" in resp.json["processes"][0]
        assert "id" in resp.json["processes"][0]

        respModules = [i["id"] for i in resp.json["processes"]]

        for i in someVirtualModules:
            assert i in respModules

    def test_filter_list_modules_get_admin_1(self):
        """Test HTTP GET /modules with filter as admin"""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?category=slope",
            headers=self.admin_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        assert len(resp.json["processes"]) == 2

    def test_filter_list_modules_get_admin_2(self):
        """Test HTTP GET /modules with filter as admin"""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?category=slope&tag=grass",
            headers=self.admin_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        assert len(resp.json["processes"]) == 2

    def test_filter_list_modules_get_admin_3(self):
        """Test HTTP GET /grass_modules with filter"""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?record=full&family=ps",
            headers=self.admin_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        assert len(resp.json["processes"]) == 1
        assert resp.json["processes"][0]["categories"] != 0
        assert resp.json["processes"][0]["parameters"] != 0

    def test_filter_list_modules_get_user_1(self):
        """Test HTTP GET /modules with filter"""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?tag=actinia&category=grass",
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons. Both importer and exporter are
        # whitelisted and should be found
        assert len(resp.json["processes"]) == 2

    def test_filter_list_modules_get_user_2(self):
        """
        Test HTTP GET /modules with filter as user.
        actinia modules are not whitelisted but available by default.
        """
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?tag=actinia", headers=self.user_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons. If kvdb db is empty, it should include
        # all pc_templates from templates/pc_templates including subdirs and
        # importer and exporter.
        num_of_actinia_modules = len(
            [
                x
                for x in resp.json["processes"]
                if x["id"] not in ["exporter", "importer"]
            ]
        )
        assert num_of_actinia_modules >= 7

    def test_filter_list_modules_get_user_3(self):
        """
        Test HTTP GET /modules with filter.
        global actinia modules can be used by any user.
        """
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?tag=actinia&category=global-template",
            headers=self.user_auth_header,
        )
        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        # WARNING: this depends on existing GRASS GIS modules and possible
        # installed GRASS GIS Addons
        assert len(resp.json["processes"]) >= 7

    def test_filter_list_modules_get_restricted_user_1(self):
        """Test HTTP GET /modules with filter as restricted user."""
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?tag=grass",
            headers=self.restricted_user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert len(resp.json["processes"]) == 3

    def test_filter_list_modules_get_restricted_user_2(self):
        """
        Test HTTP GET /modules with filter as restricted user.
        actinia modules are available by default.
        """
        respStatusCode = 200
        resp = self.app.get(
            URL_PREFIX + "/modules?tag=actinia",
            headers=self.restricted_user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        num_of_actinia_modules = len(
            [
                x
                for x in resp.json["processes"]
                if x["id"] not in ["exporter", "importer"]
            ]
        )
        assert num_of_actinia_modules >= 7


for i in someVirtualModules:
    """
    Test HTTP GET /modules/<module> for file based templates and GRASS GIS
    modules in loop for all examples in someVirtualModules above and compares
    response to file
    """
    # create method for every actinia-module to have a better overview in
    # test summary
    def_name = "test_describe_virtual_module_get_" + i
    compare_module_to_file.__defaults__ = (
        "modules",
        i,
    )
    setattr(VirtualModulesTest, def_name, compare_module_to_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Add endpoints to flask app with endpoint definitions and routes
"""

__author__ = "Carmen Tawalika, Anika Weinmann"
__copyright__ = "2018-2025 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


# from flask import current_app, send_from_directory
# import werkzeug

# from actinia_module_plugin.resources.logging import log
from flask_restful_swagger_2 import Resource

from actinia_module_plugin.api.modules.grass import ListModules
from actinia_module_plugin.api.modules.grass import DescribeModule
from actinia_module_plugin.api.modules.actinia import ListProcessChainTemplates
from actinia_module_plugin.api.modules.actinia import (
    DescribeProcessChainTemplate,
)
from actinia_module_plugin.api.modules.actinia_process import (
    ProcessActiniaModule,
)
from actinia_module_plugin.api.modules.combined import ListVirtualModules
from actinia_module_plugin.api.modules.combined import DescribeVirtualModule
from actinia_module_plugin.api.processing import (
    GdiAsyncEphemeralExportResource,
    GdiAsyncPersistentResource,
)

from actinia_module_plugin.api.actinia_templates import ActiniaTemplate
from actinia_module_plugin.api.actinia_templates import ActiniaTemplateId


def get_endpoint_class_name(
    endpoint_class: Resource,
    projects_url_part: str = "projects",
) -> str:
    """Create the name for the given endpoint class."""
    endpoint_class_name = endpoint_class.__name__.lower()
    if projects_url_part != "projects":
        name = f"{endpoint_class_name}_{projects_url_part}"
    else:
        name = endpoint_class_name
    return name


def create_project_endpoints(apidoc, projects_url_part="projects"):
    """
    Function to add resources with "projects" inside the endpoint url.

    Args:
        apidoc (flask_restful_swagger_2.Api): Flask api
        projects_url_part (str): The name of the projects inside the endpoint
                                 URL; to add deprecated location endpoints set
                                 it to "locations"
    """

    apidoc.add_resource(
        GdiAsyncEphemeralExportResource,
        f"/{projects_url_part}/<string:project_name>/processing_export",
        endpoint=get_endpoint_class_name(
            GdiAsyncEphemeralExportResource, projects_url_part
        ),
    )
    apidoc.add_resource(
        GdiAsyncPersistentResource,
        f"/{projects_url_part}/<string:project_name>/mapsets/"
        "<string:mapset_name>/processing",
        endpoint=get_endpoint_class_name(
            GdiAsyncPersistentResource, projects_url_part
        ),
    )


def create_endpoints(flask_api):
    # app = flask_api.app
    apidoc = flask_api

    # @app.route('/')
    # def index():
    #     try:
    #         # flask cannot reach out of current_app (which is actinia_core)
    #         return current_app.send_static_file('index.html')
    #     except werkzeug.exceptions.NotFound:
    #         log.debug('No index.html found. Serving backup.')
    #         return ("""<h1 style='color:red'>actinia</h1>
    #             <a href="swagger.json">API docs</a>""")
    #
    # @app.route('/<path:filename>')
    # def static_content(filename):
    #     # WARNING: all content from folder "static" will be accessible!
    #     return send_from_directory(app.static_folder, filename)

    apidoc.add_resource(ListModules, "/grass_modules")
    apidoc.add_resource(DescribeModule, "/grass_modules/<grassmodule>")

    apidoc.add_resource(ListProcessChainTemplates, "/actinia_modules")
    apidoc.add_resource(
        DescribeProcessChainTemplate, "/actinia_modules/<actiniamodule>"
    )
    apidoc.add_resource(
        ProcessActiniaModule, "/actinia_modules/<actiniamodule>/process"
    )

    apidoc.add_resource(ListVirtualModules, "/modules")
    apidoc.add_resource(DescribeVirtualModule, "/modules/<module>")

    # add deprecated location and project endpoints
    create_project_endpoints(apidoc)
    create_project_endpoints(apidoc, projects_url_part="locations")

    apidoc.add_resource(ActiniaTemplate, "/actinia_templates")
    apidoc.add_resource(ActiniaTemplateId, "/actinia_templates/<template_id>")

    # apidoc.add_resource(Actinia, '/actinia/<path:actinia_path>')
    # allows "/" inside variable



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2023-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test the usage of environment variables
"""

__license__ = "Apache-2.0"
__author__ = "Anika Weinmann"
__copyright__ = "Copyright 2023, mundialis"


from flask import Response
import json
from time import sleep

from actinia_api import URL_PREFIX

from testsuite import (
    ActiniaTestCase,
    check_started_process,
)

global allTemplatesCount
global templateUUID


class ActiniaTestEnvValues(ActiniaTestCase):
    def test_env_values_get(self):
        """Test Usage of environment values inside a template"""

        respStatusCode = 200
        msg = "Default value exist for this installation."

        resp = self.app.get(
            f"{URL_PREFIX}/actinia_modules/use_env_value",
            headers=self.user_auth_header,
        )
        assert isinstance(resp, Response)
        assert (
            resp.status_code == respStatusCode
        ), f"Status code is {resp.status_code} insted of {respStatusCode}"
        params = {
            p["name"]: [p["optional"], p["description"]]
            for p in resp.json["parameters"]
        }
        assert "env_raster" in params, "'env_raster' not in params"
        assert params["env_raster"][
            0
        ], f"{params['env_raster'][0]} is not True"

        assert msg in params["env_raster"][1], "'msg' not in env_raster"

    def test_env_values_processing(self):
        """Test usage of envrionment values in processing procedure"""

        respStatusCode = 200
        json_path = "tests/resources/processing/env_var.json"
        url_path = f"/{self.project_url_part}/nc_spm_08/processing_export"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response), "'resp' is not of class Response"
        assert (
            resp.status_code == respStatusCode
        ), f"Status code is {resp.status_code} insted of {respStatusCode}"
        assert hasattr(resp, "json"), "'resp' has no attribute 'json'"

        check_started_process(self, resp)

        # poll status until finished
        status = resp.json["status"]
        while status not in ["error", "finished"]:
            sleep(3)
            resp = self.app.get(
                resp.json["urls"]["status"], headers=self.user_auth_header
            )
            status = resp.json["status"]

        # check if parameter is set
        resp.json["process_chain_list"]
        params = {
            param["param"]: param["value"]
            for param in resp.json["process_chain_list"][0]["list"][0][
                "inputs"
            ]
        }
        assert "type" in params, "Parameter 'type' is set"
        assert params["type"] == "raster", "Parameter 'type' is not 'raster'"

    def test_env_values_processing_overwrite(self):
        """
        Test usage overwriting the envrionment values in processing
        procedure
        """

        respStatusCode = 200
        json_path = "tests/resources/processing/env_var_overwrite.json"
        url_path = f"/{self.project_url_part}/nc_spm_08/processing_export"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

        # poll status until finished
        status = resp.json["status"]
        while status not in ["error", "finished"]:
            sleep(3)
            resp = self.app.get(
                resp.json["urls"]["status"], headers=self.user_auth_header
            )
            status = resp.json["status"]

        # check if parameter is set
        resp.json["process_chain_list"]
        params = {
            param["param"]: param["value"]
            for param in resp.json["process_chain_list"][0]["list"][0][
                "inputs"
            ]
        }
        assert "type" in params, "Parameter 'type' is set"
        assert params["type"] == "vector", "Parameter 'type' is not 'vector'"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test Module Lists and Self-Description
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"

import json
from flask import Response

from actinia_api import URL_PREFIX

from testsuite import (
    ActiniaTestCase,
    import_user_template,
    delete_user_template,
)


class ActiniaModulesTest(ActiniaTestCase):
    def test_read_user_module_get(self):
        """Test HTTP GET /actinia_modules/<module> for kvdb based templates"""
        import_user_template(self, "user_point_in_polygon")

        respStatusCode = 200
        json_path = "tests/resources/actinia_modules/point_in_polygon.json"
        url_path = "/actinia_modules/user_point_in_polygon"

        with open(json_path) as file:
            expectedResp = json.load(file)
        expectedResp["id"] = "user_point_in_polygon"
        curr = expectedResp["categories"]
        new = [s.replace("global-template", "user-template") for s in curr]
        expectedResp["categories"] = new

        resp = self.app.get(
            URL_PREFIX + url_path, headers=self.user_auth_header
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert resp.json == expectedResp

        delete_user_template(self, "user_point_in_polygon")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Process Chain Template Management
File reader for global templates
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"
__maintainer__ = "Carmen Tawalika"


from actinia_module_plugin.core.common import filter_func
from actinia_module_plugin.core.modules.actinia_common import render_template
from actinia_module_plugin.resources.templating import pcTplEnv


def getAll():
    """
    This method creates a list of all global templates stored as file
    to return the list to the api method listing all templates.
    """
    tpl_list = pcTplEnv.list_templates(filter_func=filter_func)
    tpl_list = [i.replace(".json", "").split("/")[-1] for i in tpl_list]
    return tpl_list


def getTemplate(template_id):
    """
    This method uses other core methods to return a rendered template stored
    as file to return it to the api method showing a certain template.
    """
    return render_template(template_id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2016-2025, Sören Gebbert and mundialis GmbH & Co. KG

SPDX-License-Identifier: GPL-3.0-or-later

Code based on actinia_core: https://github.com/actinia-org/actinia-core
actinia-core - an open source REST API for scalable, distributed, high
performance processing of geographical data that uses GRASS GIS for
computational tasks. For details, see https://actinia.mundialis.de/

Extension of actinia_cores ephemeral_processing_with_export
AsyncEphemeralExportResource to include process chain templates
"""

__license__ = "GPLv3"
__author__ = "Anika Bettge, Sören Gebbert"
__copyright__ = "Copyright 2016-2025, Sören Gebbert, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


from copy import deepcopy
import pickle

from flask import jsonify, make_response
from flask_restful_swagger_2 import swagger
from actinia_core.core.common.kvdb_interface import enqueue_job
from actinia_core.models.response_models import create_response_from_model
from actinia_core.processing.common.ephemeral_processing_with_export import (
    start_job as start_job_ephemeral_processing_with_export,
)
from actinia_core.processing.common.persistent_processing import (
    start_job as start_job_persistent_processing,
)
from actinia_rest_lib.resource_base import ResourceBase
from actinia_api.swagger2.actinia_core.apidocs.ephemeral_processing_with_export import (
    post_doc as SCHEMA_DOC_EPHEMERAL_PROCESSING_WITH_EXPORT,
)
from actinia_api.swagger2.actinia_core.apidocs.persistent_processing import (
    post_doc as SCHEMA_DOC_PERSISTENT_PROCESSING,
)

from actinia_module_plugin.core.common import fillTemplateFromProcessChain
from actinia_module_plugin.core.modules.actinia_global_templates import (
    createProcessChainTemplateListFromFileSystem,
)
from actinia_module_plugin.core.modules.actinia_user_templates import (
    createProcessChainTemplateListFromKvdb,
)
from actinia_module_plugin.core.modules.grass import createModuleList
from actinia_module_plugin.core.processing import (
    build_kwargs_for_template_rendering,
)


def log_error_to_resource_logger(self, msg, rdc):
    """
    Logs error which occurs during translation of actinia-module to process
    chain. This case is not handled by EphemeralProcessing from
    actinia-processing-lib as _send_resource_error method is used if the error
    occurs during processing.
    Here a process (g.search.modules) by createModuleList was already processed
    and the actual process chain was not passed to actinia-core, so the error
    occurs before processing.
    """

    data = create_response_from_model(
        user_id=self.user_id,
        resource_id=self.resource_id,
        status="error",
        orig_time=self.orig_time,
        orig_datetime=self.orig_datetime,
        message=msg,
        http_code=400,
        status_url=self.status_url,
        api_info=self.api_info,
    )
    self.resource_logger.commit(
        user_id=self.user_id,
        resource_id=self.resource_id,
        iteration=1,
        document=data,
        expiration=rdc.config.KVDB_RESOURCE_EXPIRE_TIME,
    )


def set_actinia_modules(
    self, rdc, pc_list, grass_module_list, actinia_module_list
):
    new_pc = []
    for module in pc_list:
        if "module" in module:
            name = module["module"]
            if name in ["importer", "exporter"]:
                new_pc.append(module)
            elif name in grass_module_list:
                new_pc.append(module)
            elif name in actinia_module_list:
                kwargs = build_kwargs_for_template_rendering(module)
                module_pc = fillTemplateFromProcessChain(
                    module["module"],
                    kwargs,
                )["list"]
                if isinstance(module_pc, str):
                    # then return value is a missing attribute
                    msg = (
                        "Required parameter '%s' missing in actinia-module "
                        " '%s'." % (module_pc, name)
                    )
                    log_error_to_resource_logger(self, msg, rdc)
                    return
                elif module_pc is None:
                    msg = "Invalid request for %s" % (name)
                    log_error_to_resource_logger(self, msg, rdc)
                    return
                else:
                    ac_module_pc = set_actinia_modules(
                        self,
                        rdc,
                        module_pc,
                        grass_module_list,
                        actinia_module_list,
                    )
                    new_pc.extend(ac_module_pc)

            else:
                msg = (
                    "Module %s is not of type importer, exporter, "
                    "grass-module or an actinia-module." % name
                )
                log_error_to_resource_logger(self, msg, rdc)
                return
        else:
            new_pc.append(module)
    return new_pc


def preprocess_build_pc_and_enqueue(self, preprocess_kwargs, start_job):
    """
    This method looks up the lists of GRASS GIS and actinia modules to
    parse the incoming process chain. If an actinia-module is found, it is
    translated to a process chain via the stored template. The process chain is
    then passed to actinia-core.
    """

    # get grass and actinia module lists
    module_list = createModuleList(self)
    pc_global_list = createProcessChainTemplateListFromFileSystem()
    pc_user_list = createProcessChainTemplateListFromKvdb()
    grass_module_list = []
    actinia_module_list = []

    for module in module_list:
        grass_module_list.append(module["id"])

    for module in pc_global_list:
        actinia_module_list.append(module["id"])

    for module in pc_user_list:
        actinia_module_list.append(module["id"])

    # run preprocess again after createModuleList
    rdc = self.preprocess(**preprocess_kwargs)

    if rdc:
        rdc.set_storage_model_to_file()

        new_pc = set_actinia_modules(
            self,
            rdc,
            rdc.request_data["list"],
            grass_module_list,
            actinia_module_list,
        )
        rdc.request_data["list"] = new_pc

        enqueue_job(self.job_timeout, start_job, rdc)


class GdiAsyncEphemeralExportResource(ResourceBase):
    """
    This class represents a resource that runs asynchronous processing tasks in
    a temporary mapset and exports the computed results as geotiff files.
    """

    def __init__(self):
        ResourceBase.__init__(self)

    @swagger.doc(deepcopy(SCHEMA_DOC_EPHEMERAL_PROCESSING_WITH_EXPORT))
    def post(self, project_name):
        """
        Execute a user defined process chain in an ephemeral project/mapset
        and store the processing results for download.
        """

        preprocess_kwargs = {}
        preprocess_kwargs["has_json"] = True
        preprocess_kwargs["project_name"] = project_name

        start_job = start_job_ephemeral_processing_with_export

        preprocess_build_pc_and_enqueue(self, preprocess_kwargs, start_job)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)


class GdiAsyncPersistentResource(ResourceBase):
    """
    This class represents a resource that runs asynchronous processing tasks in
    a persistent mapset.
    """

    def __init__(self):
        ResourceBase.__init__(self)

    @swagger.doc(deepcopy(SCHEMA_DOC_PERSISTENT_PROCESSING))
    def post(self, project_name, mapset_name):
        """
        Execute a user defined process chain that creates a new mapset or
        runs in an existing one.
        """

        preprocess_kwargs = {}
        preprocess_kwargs["has_json"] = True
        preprocess_kwargs["project_name"] = project_name
        preprocess_kwargs["mapset_name"] = mapset_name

        start_job = start_job_persistent_processing

        preprocess_build_pc_and_enqueue(self, preprocess_kwargs, start_job)

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

GRASS GIS module viewer

* List all modules
* Describe single module
"""

__license__ = "Apache-2.0"
__author__ = "Anika Weinmann, Carmen Tawalika, Julia Haas"
__copyright__ = "Copyright 2019 - 2022, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis GmbH & Co. KG"


from flask import jsonify, make_response, request
from flask_restful_swagger_2 import swagger
from actinia_rest_lib.resource_base import ResourceBase

from actinia_module_plugin.apidocs import modules
from actinia_module_plugin.core.filter import filter
from actinia_module_plugin.core.modules.grass import createModuleList
from actinia_module_plugin.core.modules.grass import createModuleUserList
from actinia_module_plugin.core.modules.grass import createGrassModule
from actinia_module_plugin.core.modules.grass import createFullModuleList
from actinia_module_plugin.core.modules.grass import installGrassAddon
from actinia_module_plugin.core.modules.accessible_modules_kvdb_interface import (
    addGrassAddonToModuleListKvdb,
)
from actinia_module_plugin.model.modules import ModuleList
from actinia_module_plugin.model.responseModels import (
    SimpleStatusCodeResponseModel,
)


class ListModules(ResourceBase):
    """List all GRASS modules"""

    @swagger.doc(modules.listModules_get_docs)
    def get(self):
        """Get a list of all GRASS GIS modules."""

        module_list = createModuleList(self)
        if self.user_role == "user" or self.user_role == "guest":
            # admins have access to all modules
            user_list = createModuleUserList(self)
            final_list = [m for m in module_list if m["id"] in user_list]
        else:
            final_list = module_list
        final_list = filter(final_list)

        if "record" in request.args:
            if request.args["record"] == "full":
                final_list = createFullModuleList(self, final_list)

        return make_response(
            jsonify(ModuleList(status="success", processes=final_list)), 200
        )


class DescribeModule(ResourceBase):
    """
    Definition for endpoint @app.route('grass_modules/<grassmodule>') to
    desctibe one module

    Contains HTTP GET endpoint
    Contains swagger documentation
    """

    @swagger.doc(modules.describeGrassModule_get_docs)
    def get(self, grassmodule):
        """Describe a GRASS GIS module."""

        try:
            grass_module = createGrassModule(self, grassmodule)
            return make_response(jsonify(grass_module), 200)
        except Exception:
            res = jsonify(
                SimpleStatusCodeResponseModel(
                    status=404,
                    message='Error looking for module "' + grassmodule + '".',
                )
            )
            return make_response(res, 404)

    def post(self, grassmodule):
        """Install an official GRASS GIS Addon."""

        response = installGrassAddon(self, grassmodule)

        if response["status"] == "finished":
            addGrassAddonToModuleListKvdb(self, grassmodule)
            msg = "Successfully installed GRASS addon " + grassmodule + "."
            status_code = 201
        else:
            msg = "Error installing GRASS addon " + grassmodule + "."
            status_code = 400

        res = jsonify(
            SimpleStatusCodeResponseModel(status=status_code, message=msg)
        )
        return make_response(res, status_code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2016-2025 Sören Gebbert & mundialis GmbH & Co. KG

SPDX-License-Identifier: GPL-3.0-or-later

Code based on actinia_core: https://github.com/actinia-org/actinia-core
actinia-core - an open source REST API for scalable, distributed, high
performance processing of geographical data that uses GRASS GIS for
computational tasks. For details, see https://actinia.mundialis.de/

Base class for GRASS GIS REST API tests
"""

__license__ = "GPLv3"
__author__ = "Carmen Tawalika, Sören Gebbert"
__copyright__ = "Copyright 2016-2025, Sören Gebbert, mundialis GmbH & Co. KG"
__maintainer__ = "mundialis"


import base64
from copy import deepcopy
import json
import unittest

import pwgen
from werkzeug.datastructures import Headers

from actinia_core.endpoints import create_endpoints
from actinia_core.core.common.kvdb_interface import connect, disconnect
from actinia_core.core.common.app import flask_app, URL_PREFIX
from actinia_core.core.common.config import global_config
from actinia_core.core.common.process_queue import create_process_queue
from actinia_core.models.response_models import ProcessingResponseModel
from actinia_core.core.common.user import ActiniaUser
from actinia_core.version import init_versions, G_VERSION

# actinia-module-plugin endpoints are included as defined in actinia_core
# config
create_endpoints()


class ActiniaTestCase(unittest.TestCase):
    # guest = None
    # admin = None
    # superadmin = None
    user = None
    auth_header = {}
    users_list = []
    project_url_part = "projects"

    # set project_url_part to "locations" if GRASS GIS version < 8.4
    init_versions()
    grass_version_s = G_VERSION["version"]
    grass_version = [int(item) for item in grass_version_s.split(".")[:2]]
    if grass_version < [8, 4]:
        project_url_part = "locations"

    def setUp(self):
        """Overwrites method setUp from unittest.TestCase class"""

        self.app_context = flask_app.app_context()
        self.app_context.push()
        # from http://flask.pocoo.org/docs/0.12/api/#flask.Flask.test_client:
        # Note that if you are testing for assertions or exceptions in your
        # application code, you must set app.testing = True in order for the
        # exceptions to propagate to the test client.  Otherwise, the exception
        # will be handled by the application (not visible to the test client)
        # and the only indication of an AssertionError or other exception will
        # be a 500 status code response to the test client.
        flask_app.testing = True
        self.app = flask_app.test_client()

        # Start and connect the kvdb interface
        kvdb_args = (
            global_config.KVDB_SERVER_URL,
            global_config.KVDB_SERVER_PORT,
        )
        if (
            global_config.KVDB_SERVER_PW
            and global_config.KVDB_SERVER_PW is not None
        ):
            kvdb_args = (*kvdb_args, global_config.KVDB_SERVER_PW)
        connect(*kvdb_args)

        # create test user for roles user (more to come)
        accessible_datasets = {
            "nc_spm_08": ["PERMANENT", "user1", "modis_lst"]
        }

        accessible_modules = deepcopy(global_config.MODULE_ALLOW_LIST)
        accessible_modules.extend(["v.db.addcolumn", "v.what.vect"])
        password = pwgen.pwgen()
        (
            self.user_id,
            self.user_group,
            self.user_auth_header,
        ) = self.createUser(
            name="user",
            role="user",
            password=password,
            process_num_limit=30,
            process_time_limit=4,
            accessible_datasets=accessible_datasets,
            accessible_modules=accessible_modules,
        )
        (
            self.restricted_user_id,
            self.restricuted_user_group,
            self.restricted_user_auth_header,
        ) = self.createUser(
            name="user2",
            role="user",
            password=password,
            process_num_limit=30,
            process_time_limit=4,
            accessible_datasets=accessible_datasets,
            accessible_modules=["v.db.select", "importer", "r.mapcalc"],
        )
        (
            self.admin_id,
            self.admin_group,
            self.admin_auth_header,
        ) = self.createUser(
            name="admin",
            role="admin",
            password=password,
            process_num_limit=30,
            process_time_limit=4,
            accessible_datasets=accessible_datasets,
        )

        # create process queue
        create_process_queue(config=global_config)

    def tearDown(self):
        """Overwrites method tearDown from unittest.TestCase class"""

        self.app_context.pop()

        # remove test user; disconnect kvdb
        for user in self.users_list:
            user.delete()
        disconnect()

    def createUser(
        self,
        name="guest",
        role="guest",
        group="group",
        password="abcdefgh",
        accessible_datasets=None,
        accessible_modules=global_config.MODULE_ALLOW_LIST,
        process_num_limit=1000,
        process_time_limit=6000,
        cell_limit=1100000000,
    ):
        auth = bytes("%s:%s" % (name, password), "utf-8")
        # We need to create an HTML basic authorization header
        self.auth_header[role] = Headers()
        self.auth_header[role].add(
            "Authorization", "Basic " + base64.b64encode(auth).decode()
        )

        # Make sure the user database is empty
        user = ActiniaUser(name)
        if user.exists():
            user.delete()
        # Create a user in the database
        user = ActiniaUser.create_user(
            name,
            group,
            password,
            user_role=role,
            accessible_datasets=accessible_datasets,
            accessible_modules=accessible_modules,
            process_num_limit=process_num_limit,
            process_time_limit=process_time_limit,
            cell_limit=cell_limit,
        )
        user.add_accessible_modules(["uname", "sleep"])
        self.users_list.append(user)

        return name, group, self.auth_header[role]


# import unittest
# @unittest.skip("compare response to file")
def compare_module_to_file(self, uri_path="modules", module=None):
    """Compares response of API call to file"""
    # Won't run with module=None but ensures, that "passing of arguments"
    # below is successful.

    resp = self.app.get(
        URL_PREFIX + "/" + uri_path + "/" + module,
        headers=self.user_auth_header,
    )
    respStatusCode = 200
    assert hasattr(resp, "json")
    currentResp = resp.json

    with open("tests/resources/actinia_modules/" + module + ".json") as file:
        expectedResp = json.load(file)

    assert resp.status_code == respStatusCode
    assert currentResp == expectedResp


def import_user_template(testCase, name):
    """Imports user template to kvdb database (Create)"""
    json_path = "tests/resources/actinia_templates/" + name + ".json"
    with open(json_path) as file:
        pc_template = json.load(file)
    resp = testCase.app.post(
        URL_PREFIX + "/actinia_templates",
        headers=testCase.user_auth_header,
        data=json.dumps(pc_template),
        content_type="application/json",
    )
    assert resp.status == "201 CREATED"


def delete_user_template(testCase, name):
    """Deletes user template from kvdb database (Delete) if exists"""
    resp = testCase.app.get(
        URL_PREFIX + "/actinia_templates/" + name,
        headers=testCase.user_auth_header,
    )
    if resp.status_code != 404:
        resp = testCase.app.delete(
            URL_PREFIX + "/actinia_templates/" + name,
            headers=testCase.user_auth_header,
        )
        assert resp.status_code == 200


def check_started_process(testCase, resp):
    """Checks response of started process - TODO: can be enhanced"""
    if isinstance(resp.json["process_results"], dict):
        resp_json = deepcopy(resp.json)
        del resp_json["process_results"]
        resp_json["process_results"] = str(resp.json["process_results"])
    resp_class = ProcessingResponseModel(**resp_json)
    assert resp_class["status"] == "accepted"
    status_url = resp_class["urls"]["status"]

    # poll status_url
    # TODO: status stays in accepted
    status_resp = testCase.app.get(
        status_url, headers=testCase.user_auth_header
    )
    assert status_resp.json["urls"]["status"] == status_url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

actinia-module viewer
Templates can be stored file based and in kvdb

* List all actinia-modules
* Describe single actinia-module
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2025, mundialis"
__maintainer__ = "Carmen Tawalika"


from flask import jsonify, make_response
from jinja2.exceptions import TemplateNotFound
import pickle

from actinia_core.core.common.kvdb_interface import enqueue_job
from actinia_core.processing.common.ephemeral_processing_with_export import (
    start_job as start_job_ephemeral_processing_with_export,
)
from actinia_rest_lib.resource_base import ResourceBase

from actinia_module_plugin.core.common import (
    fillTemplateFromProcessChain,
)
from actinia_module_plugin.core.modules.actinia_common import (
    createActiniaModule,
)
from actinia_module_plugin.core.common import (
    get_user_template_source,
    get_global_template_source,
)
from actinia_module_plugin.core.template_parameters import (
    get_template_undef,
)


def preprocess_load_tpl_and_enqueue(
    self, preprocess_kwargs, start_job, actiniamodule
):
    """
    This method looks up the stored process chain template.
    Template values are filled according to input values.
    The process chain is then passed to actinia-core.
    """

    # run preprocess again after createModuleList
    rdc = self.preprocess(**preprocess_kwargs)

    if rdc:
        rdc.set_storage_model_to_file()

        tpl_source = get_user_template_source(
            actiniamodule
        ) or get_global_template_source(actiniamodule)
        undef = get_template_undef(tpl_source)

        # TODO parse request data when schema is defined
        # Might be close to OGC API processes
        kwargs = {}
        kwargs[next(iter(undef))] = rdc.request_data

        new_pc = fillTemplateFromProcessChain(actiniamodule, kwargs)
        rdc.request_data = new_pc

        enqueue_job(self.job_timeout, start_job, rdc)


class ProcessActiniaModule(ResourceBase):
    """
    Process process chain template as actinia-module.

    Contains HTTP POST endpoint
    Contains swagger documentation
    """

    # TODO: Define input
    # @swagger.doc(modules.processActiniaModule_post_docs)
    def post(self, actiniamodule):
        """Process an actinia module (process chain template)."""

        preprocess_kwargs = {}
        preprocess_kwargs["has_json"] = True
        # TODO: Currently no project can be read out of request body.
        # Instead it will take the first project listed in actinia module
        # To be sure only write a single project inside template.
        try:
            virtual_module = createActiniaModule(self, actiniamodule)
        except TemplateNotFound:
            return make_response(
                jsonify(
                    {
                        "code": 404,
                        "message": f"Module '{actiniamodule}' not found.",
                    }
                ),
                404,
            )
        preprocess_kwargs["project_name"] = virtual_module["projects"][0]

        start_job = start_job_ephemeral_processing_with_export

        preprocess_load_tpl_and_enqueue(
            self, preprocess_kwargs, start_job, actiniamodule
        )

        html_code, response_model = pickle.loads(self.response_data)
        return make_response(jsonify(response_model), html_code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Configuration file
"""

__author__ = "Carmen Tawalika"
__copyright__ = "2018-2021 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


import configparser
import glob
from pathlib import Path
import os

# config can be overwritten by mounting *.ini files into folders inside
# the config folder.
DEFAULT_CONFIG_PATH = "config"
CONFIG_FILES = [
    str(f) for f in Path(DEFAULT_CONFIG_PATH).glob("**/*.ini") if f.is_file()
]
GENERATED_CONFIG = DEFAULT_CONFIG_PATH + "/actinia-module-plugin.cfg"


class PCTEMPLATECONFIG:
    """Default path for 'templates/pc_templates'"""

    pathfile = "/src/actinia-module-plugin/config/templates/pc_templates"


class LOGCONFIG:
    """Default config for logging"""

    logfile = "actinia-module-plugin.log"
    level = "DEBUG"
    type = "stdout"


class Configfile:
    def __init__(self):
        """
        This class will overwrite the config classes above when config files
        named DEFAULT_CONFIG_PATH/**/*.ini exist.
        On first import of the module it is initialized.
        """
        from actinia_module_plugin.resources.logging import log

        config = configparser.ConfigParser()
        config.read(CONFIG_FILES)

        if len(config) <= 1:
            log.info("Could not find any config file, using default values.")
            return
        log.info("Loading config files: " + str(CONFIG_FILES) + " ...")

        with open(GENERATED_CONFIG, "w") as configfile:
            config.write(configfile)
        log.debug("Configuration written to " + GENERATED_CONFIG)

        # LOGGING
        if config.has_section("LOGCONFIG"):
            if config.has_option("LOGCONFIG", "logfile"):
                LOGCONFIG.logfile = config.get("LOGCONFIG", "logfile")
            if config.has_option("LOGCONFIG", "level"):
                LOGCONFIG.level = config.get("LOGCONFIG", "level")
            if config.has_option("LOGCONFIG", "type"):
                LOGCONFIG.type = config.get("LOGCONFIG", "type")

        # TEMPLATE PATH
        if os.getenv("PCTEMPLATES") is not None:
            PCTEMPLATECONFIG.pathfile = os.getenv("PCTEMPLATES")
        elif config.has_section("PCTEMPLATECONFIG"):
            if config.has_option("PCTEMPLATECONFIG", "pathfile"):
                PCTEMPLATECONFIG.pathfile = config.get(
                    "PCTEMPLATECONFIG", "pathfile"
                )


init = Configfile()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2018-2021 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

actinia-module + grass-module viewer
Templates can be stored file based and in kvdb

* List all GRASS GIS modules and actinia-modules
* Describe single module
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2019, mundialis"
__maintainer__ = "Carmen Tawalika"


from flask import jsonify, make_response, request
from flask_restful_swagger_2 import swagger
from actinia_rest_lib.resource_base import ResourceBase

from actinia_module_plugin.apidocs import modules
from actinia_module_plugin.core.filter import filter
from actinia_module_plugin.core.modules.actinia_global_templates import (
    createProcessChainTemplateListFromFileSystem,
)
from actinia_module_plugin.core.modules.actinia_user_templates import (
    createProcessChainTemplateListFromKvdb,
)
from actinia_module_plugin.core.modules.actinia_common import (
    createActiniaModule,
)
from actinia_module_plugin.core.modules.grass import createModuleList
from actinia_module_plugin.core.modules.grass import createModuleUserList
from actinia_module_plugin.core.modules.grass import createGrassModule
from actinia_module_plugin.core.modules.grass import createFullModuleList
from actinia_module_plugin.model.modules import ModuleList
from actinia_module_plugin.model.responseModels import (
    SimpleStatusCodeResponseModel,
)


class ListVirtualModules(ResourceBase):
    """List all GRASS GIS modules and process chain templates"""

    @swagger.doc(modules.listModules_get_docs)
    def get(self):
        """Get a list of all modules."""

        grass_list = createModuleList(self)
        if self.user_role == "user" or self.user_role == "guest":
            # admins have access to all modules
            user_list = createModuleUserList(self)
            final_grass_list = [m for m in grass_list if m["id"] in user_list]
        else:
            final_grass_list = grass_list
        final_grass_list = filter(final_grass_list)

        if "record" in request.args:
            if request.args["record"] == "full":
                final_grass_list = createFullModuleList(self, final_grass_list)

        pc_list_fs = createProcessChainTemplateListFromFileSystem()
        pc_list_kvdb = createProcessChainTemplateListFromKvdb()
        module_list = final_grass_list + pc_list_fs + pc_list_kvdb

        module_list = filter(module_list)

        return make_response(
            jsonify(ModuleList(status="success", processes=module_list)), 200
        )


class DescribeVirtualModule(ResourceBase):
    """
    Describe module or process chain template

    Contains HTTP GET endpoint
    Contains swagger documentation
    """

    @swagger.doc(modules.describeModule_get_docs)
    def get(self, module):
        """Describe a module."""

        try:
            try:
                virtual_module = createGrassModule(self, module)
            except Exception:
                virtual_module = createActiniaModule(self, module)
            finally:
                return make_response(jsonify(virtual_module), 200)

        except Exception:
            msg = 'Error looking for module "' + module + '".'
            res = jsonify(
                SimpleStatusCodeResponseModel(status=404, message=msg)
            )
            return make_response(res, 404)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test processing
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"


from flask import Response
import json

from actinia_api import URL_PREFIX

from testsuite import (
    ActiniaTestCase,
    import_user_template,
    delete_user_template,
    check_started_process,
)


global allTemplatesCount
global templateUUID


class ActiniaProcessingTest(ActiniaTestCase):
    def test_processing(self):
        """Test Usage of user templates persistent processing"""
        import_user_template(self, "user_default_value")

        respStatusCode = 200
        json_path = "tests/resources/processing/user_default_value.json"
        url_path = (
            f"/{self.project_url_part}/nc_spm_08/mapsets/test/processing"
        )

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

        resp = self.app.delete(
            f"{URL_PREFIX}/{self.project_url_part}/nc_spm_08/mapsets/test",
            headers=self.user_auth_header,
        )

        delete_user_template(self, "user_default_value")

    def test_processing_export(self):
        """Test Usage of user templates ephemeral processing"""
        import_user_template(self, "user_point_in_polygon")

        respStatusCode = 200
        json_path = "tests/resources/processing/user_point_in_polygon.json"
        url_path = f"/{self.project_url_part}/nc_spm_08/processing_export"

        with open(json_path) as file:
            pc_template = json.load(file)

        resp = self.app.post(
            URL_PREFIX + url_path,
            headers=self.user_auth_header,
            data=json.dumps(pc_template),
            content_type="application/json",
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")

        check_started_process(self, resp)

        delete_user_template(self, "user_point_in_polygon")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2021-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Test Module Lists and Self-Description
"""

__license__ = "Apache-2.0"
__author__ = "Carmen Tawalika"
__copyright__ = "Copyright 2021, mundialis"


from flask import Response

from actinia_api import URL_PREFIX

from testsuite import ActiniaTestCase, compare_module_to_file


someActiniaModules = [
    "add_enumeration",
    "default_value",
    "nested_modules_test",
    "point_in_polygon",
    "slope_aspect",
    "vector_area",
    "index_NDVI",
    "loop_simple",
    "loop",
]


class ActiniaModulesTest(ActiniaTestCase):
    def test_list_modules_get(self):
        """Test HTTP GET /actinia_modules"""
        global someActiniaModules

        respStatusCode = 200
        resp = self.app.get(URL_PREFIX + "/actinia_modules")

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert "actinia-module" in resp.json["processes"][0]["categories"]

        respModules = [i["id"] for i in resp.json["processes"]]

        for i in someActiniaModules:
            assert i in respModules

    def test_filter_list_modules_get(self):
        """Test HTTP GET /actinia_modules with filter"""
        respStatusCode = 200
        resp = self.app.get(URL_PREFIX + "/actinia_modules?tag=global")

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")
        assert len(resp.json["processes"]) >= 7

    def test_describe_modules_not_found(self):
        """Test HTTP GET /actinia_modules/<not-existing-module>"""
        respStatusCode = 404
        resp = self.app.get(
            URL_PREFIX + "/actinia_modules/not_exist",
            headers=self.user_auth_header,
        )

        assert isinstance(resp, Response)
        assert resp.status_code == respStatusCode
        assert hasattr(resp, "json")


for i in someActiniaModules:
    """
    Test HTTP GET /actinia_modules/<module> for file based templates in loop
    for all examples in someActiniaModules above and compares response to file
    """
    # create method for every actinia-module to have a better overview in
    # test summary
    def_name = "test_describe_process_chain_template_get_" + i
    compare_module_to_file.__defaults__ = (
        "actinia_modules",
        i,
    )
    setattr(ActiniaModulesTest, def_name, compare_module_to_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
SPDX-FileCopyrightText: (c) 2019-2025 by mundialis GmbH & Co. KG

SPDX-License-Identifier: Apache-2.0

Model classes for grassmodule and process chain template object
"""

__author__ = "Anika Bettge, Carmen Tawalika"
__copyright__ = "2019-2025 mundialis GmbH & Co. KG"
__license__ = "Apache-2.0"


import os
import json

from flask_restful_swagger_2 import Schema


script_dir = os.path.dirname(os.path.abspath(__file__))
rel_path = "../apidocs/examples/gm_describemodule_get_example_v_random.json"
abs_file_path = os.path.join(script_dir, rel_path)
with open(abs_file_path) as jsonfile:
    describemodule_get_docs_example = json.load(jsonfile)


class ModuleParameterSchema(Schema):
    """
    Schema property for Parameter
    TODO: add  {"type": "array", "items": {"type": "string"}}
    """

    type = "object"
    properties = {
        "type": {"type": "string", "description": ""},
        "subtype": {"type": "string", "description": ""},
        "enum": {
            "type": "array",
            "items": {"type": "string"},
            "description": "",
        },
    }
    description = "A schema object according to the specification of JSON "
    " Schema draft-07. Additional values for format are defined "
    " centrally in the API documentation, e.g. bbox or crs. "
    " Callback parameters are defined with the custom schema "
    " keyword parameters."


class ModuleParameter(Schema):
    """Parameters from Module"""

    type = "object"
    properties = {
        "name": {
            "type": "string",
            "description": "A unique name for the parameter. ",
        },
        "description": {
            "type": "string",
            "description": "Detailed description to fully explain the entity.",
        },
        "optional": {
            "type": "boolean",
            "description": "Determines whether this parameter is mandatory. "
            " Default: true",
        },
        "default": {
            "type": "string",
            "description": "The default value for this parameter.",
        },
        "schema": ModuleParameterSchema,
        # 'comment': {
        #     'type': 'string',
        #     'description': 'Comment for parameter.'
        # }
    }
    description = "A list of parameters that are applicable for this process."
    required = ["description", "schema", "name"]


class ModuleReturns(ModuleParameter):
    properties = ModuleParameter.properties
    required = ModuleParameter.required
    type = ModuleParameter.type
    description = "The data that is returned from this process."


class ModuleImportDescription(ModuleParameter):
    properties = ModuleParameter.properties
    required = ModuleParameter.required
    type = ModuleParameter.type
    description = "Import parameters to import data for this process."


class ModuleExportDescription(ModuleParameter):
    properties = ModuleParameter.properties
    required = ModuleParameter.required
    type = ModuleParameter.type
    description = "Export parameters to export returned data from this process"


class Module(Schema):
    """Response schema for module"""

    type = "object"
    properties = {
        "id": {
            "type": "string",
            "description": "Unique identifier of the process. ",
        },
        "summary": {
            "type": "string",
            "description": "A short summary of what the process does.",
        },
        "description": {
            "type": "string",
            "description": "Detailed description to fully explain the entity.",
        },
        "categories": {
            "type": "array",
            "items": {"type": "string"},
            "description": "A list of categories. GRASS GIS addons have the "
            'category "grass-module" and the actinia core '
            'modules are identified with "actinia-module"',
        },
        "projects": {
            "type": "array",
            "items": {"type": "string"},
            "description": "A comma separated list of GRASS GIS projects for "
            "which the process is suitable.",
        },
        "parameters": ModuleParameter,
        "returns": ModuleReturns,
        "import_descr": ModuleImportDescription,
        "export": ModuleExportDescription,
    }
    example = describemodule_get_docs_example
    required = ["id", "description"]
    # required = ["id", "description", "parameters", "returns"]


class ModuleList(Schema):
    """
    Response schema for module lists
    The answer bases on openeo v0.4: https://open-eo.github.io/openeo-api/v
    /0.4.0/apireference/#tag/Process-Discovery/paths/~1processes/get
    Differences:
    * module name is set to id
    * keywords to categories
    * description is set to required description and not to optional summary
    """

    type = "object"
    properties = {
        "status": {
            "type": "string",
            "description": "The status of the resource, values: accepted, "
            "running, finished, terminated, error",
        },
        "processes": {
            "type": "array",
            "items": Module,
            "description": "The list of modules in GRASS GIS",
        },
    }
    example = {
        "processes": [
            {
                "id": "v.random",
                "description": "Generates random 2D/3D vector points.",
                "categories": [
                    "vector",
                    "sampling",
                    "statistics",
                    "random",
                    "point pattern",
                    "stratified random sampling",
                    "level1",
                ],
            }
        ],
        "status": "success",
    }
    required = ["status", "processes"]
